@@ -87,11 +87,13 @@ pub enum SeekStep {
 
 impl SeekStep {
     #[allow(clippy::missing_panics_doc)] // const unwrap
+    #[must_use]
     pub fn default_both() -> Self {
         Self::Both(NonZeroU32::new(5).unwrap())
     }
 
     #[allow(clippy::missing_panics_doc)] // const unwrap
+    #[must_use]
     pub fn default_depends() -> Self {
         Self::Depends {
             short_tracks: NonZeroU32::new(5).unwrap(),
@@ -102,6 +104,7 @@ impl SeekStep {
     /// Get Seek Step, depending on track-length
     ///
     /// directly returns a i64, though the value is never negative returned from here
+    #[must_use]
     pub fn get_step(&self, track_len: u64) -> i64 {
         match self {
             SeekStep::Both(v) => v.get().into(),
@@ -152,6 +155,7 @@ pub enum PositionYesNo {
 
 impl PositionYesNo {
     /// Get the time before saving the track position, if enabled
+    #[must_use]
     pub fn get_time(&self, media_type: MediaType) -> Option<u64> {
         match self {
             PositionYesNo::Simple(v) => match v {
@@ -167,6 +171,7 @@ impl PositionYesNo {
     }
 
     /// Get if the current value means "it is enabled"
+    #[must_use]
     pub fn is_enabled(&self) -> bool {
         match self {
             PositionYesNo::Simple(v) => *v == PositionYesNoLower::Yes,
@@ -189,6 +194,7 @@ pub enum RememberLastPosition {
 
 impl RememberLastPosition {
     /// Get the time before saving the track position, if enabled
+    #[must_use]
     pub fn get_time(&self, media_type: MediaType) -> Option<u64> {
         match self {
             RememberLastPosition::All(v) => v.get_time(media_type),
@@ -204,6 +210,7 @@ impl RememberLastPosition {
     ///
     /// use case is in the restore of the last position
     #[allow(clippy::needless_pass_by_value)] // "MediaType" is a 1-byte copy
+    #[must_use]
     pub fn is_enabled_for(&self, media_type: MediaType) -> bool {
         match self {
             RememberLastPosition::All(v) => v.is_enabled(),
@@ -296,17 +303,19 @@ impl Default for PlayerSettings {
 /// Playlist loop modes
 #[derive(Debug, Clone, Copy, Deserialize, Serialize, Default, PartialEq, Eq)]
 #[serde(rename_all = "lowercase")]
+#[repr(u8)]
 pub enum LoopMode {
     /// Loop one track
-    Single,
+    Single = 0,
     /// Loop the entire Playlist (after last index comes the first)
     #[default]
-    Playlist,
+    Playlist = 1,
     /// Select a random track on each next track
-    Random,
+    Random = 2,
 }
 
 impl LoopMode {
+    #[must_use]
     pub fn display(self, display_symbol: bool) -> &'static str {
         if display_symbol {
             match self {
@@ -322,6 +331,23 @@ impl LoopMode {
             }
         }
     }
+
+    /// Convert the current enum variant into its number representation
+    #[must_use]
+    pub fn discriminant(&self) -> u8 {
+        (*self) as u8
+    }
+
+    /// Try to convert the input number representation to a variant
+    #[must_use]
+    pub fn tryfrom_discriminant(num: u8) -> Option<Self> {
+        Some(match num {
+            0 => Self::Single,
+            1 => Self::Playlist,
+            2 => Self::Random,
+            _ => return None,
+        })
+    }
 }
 
 /// Settings for the gRPC server (and potentially future ways to communicate)
@@ -351,7 +377,7 @@ impl From<ComSettings> for SocketAddr {
 }
 
 mod v1_interop {
-    use std::{error::Error, fmt::Display, num::TryFromIntError};
+    use std::num::TryFromIntError;
 
     use super::{
         ComSettings, LoopMode, NonZeroU32, NonZeroU8, PlayerSettings, PodcastSettings,
@@ -397,31 +423,19 @@ mod v1_interop {
         }
     }
 
-    // TODO: consider upgrading this with "thiserror"
     /// Error for when [`ServerSettings`] convertion fails
-    #[derive(Debug, Clone, PartialEq)]
+    #[derive(Debug, Clone, PartialEq, thiserror::Error)]
     pub enum ServerSettingsConvertError {
         /// Recieved a zero value expecting a non-zero value
-        ///
-        /// (old key, new key, error)
-        ZeroValue(&'static str, &'static str, TryFromIntError),
+        #[error("Zero value where expecting a non-zero value. old-config key: '{old_key}', new-config key: '{new_key}', error: {source}")]
+        ZeroValue {
+            old_key: &'static str,
+            new_key: &'static str,
+            #[source]
+            source: TryFromIntError,
+        },
     }
 
-    impl Display for ServerSettingsConvertError {
-        fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
-            // let alternate = f.alternate();
-            write!(
-                f,
-                "Failed to convert from v1 to v2 config because of {}",
-                match self {
-                    Self::ZeroValue(old_key, new_key, err) => format!("zero value where expecting a non-zero value, old-config key: '{old_key}', new-config key: '{new_key}', error: {err:#}")
-                }
-            )
-        }
-    }
-
-    impl Error for ServerSettingsConvertError {}
-
     impl TryFrom<v1::Settings> for ServerSettings {
         type Error = ServerSettingsConvertError;
 
@@ -438,19 +452,17 @@ mod v1_interop {
                         .podcast_simultanious_download
                         .clamp(0, u8::MAX as usize) as u8,
                 )
-                .map_err(|err| {
-                    ServerSettingsConvertError::ZeroValue(
-                        "podcast_simultanious_download",
-                        "podcast.concurrent_downloads_max",
-                        err,
-                    )
+                .map_err(|err| ServerSettingsConvertError::ZeroValue {
+                    old_key: "podcast_simultanious_download",
+                    new_key: "podcast.concurrent_downloads_max",
+                    source: err,
                 })?,
                 max_download_retries: value.podcast_max_retries.clamp(0, u8::MAX as usize) as u8,
                 download_dir: value.podcast_dir,
             };
 
             let player_settings = PlayerSettings {
-                music_dirs: value.music_dir.into_iter().map(Into::into).collect(),
+                music_dirs: value.music_dir,
                 // not converting old scan_depth as that is not stored in the config, but set via CLI, using default instead
                 // library_scan_depth: ScanDepth::Limited(value.max_depth_cli),
                 library_scan_depth: ScanDepth::Limited(10),
@@ -467,22 +479,18 @@ mod v1_interop {
                 random_track_quantity: NonZeroU32::try_from(
                     value.playlist_select_random_track_quantity,
                 )
-                .map_err(|err| {
-                    ServerSettingsConvertError::ZeroValue(
-                        "playlist_select_random_track_quantity",
-                        "player.random_track_quantity",
-                        err,
-                    )
+                .map_err(|err| ServerSettingsConvertError::ZeroValue {
+                    old_key: "playlist_select_random_track_quantity",
+                    new_key: "player.random_track_quantity",
+                    source: err,
                 })?,
                 random_album_min_quantity: NonZeroU32::try_from(
                     value.playlist_select_random_album_quantity,
                 )
-                .map_err(|err| {
-                    ServerSettingsConvertError::ZeroValue(
-                        "playlist_select_random_album_quantity",
-                        "player.random_album_min_quantity",
-                        err,
-                    )
+                .map_err(|err| ServerSettingsConvertError::ZeroValue {
+                    old_key: "playlist_select_random_album_quantity",
+                    new_key: "player.random_album_min_quantity",
+                    source: err,
                 })?,
             };
 
