@@ -22,7 +22,7 @@ pub struct KeysCheckError {
 
 impl Display for KeysCheckError {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
-        write!(
+        writeln!(
             f,
             "There are {} Key Conflict Errors: [",
             self.errored_keys.len()
@@ -804,12 +804,12 @@ impl Default for KeysPlaylist {
                 tuievents::KeyModifiers::SHIFT,
             )
             .into(),
-            add_random_songs: tuievents::KeyEvent::new(
+            add_random_songs: tuievents::Key::Char('s').into(),
+            add_random_album: tuievents::KeyEvent::new(
                 tuievents::Key::Char('S'),
                 tuievents::KeyModifiers::SHIFT,
             )
             .into(),
-            add_random_album: tuievents::Key::Char('s').into(),
         }
     }
 }
@@ -1234,53 +1234,38 @@ impl CheckConflict for KeysDatabase {
 }
 
 // TODO: upgrade errors with what config-key has errored
-// TODO: consider upgrading this with "thiserror"
 /// Error for when [`Key`] parsing fails
-#[derive(Debug, Clone, PartialEq)]
+#[derive(Debug, Clone, PartialEq, thiserror::Error)]
 pub enum KeyParseError {
     /// Error when either the string is empty, or only has modifiers.
     ///
     /// Listing (`key_bind`)
+    #[error("Failed to parse Key because no key was found in the mapping, input: {0:#?}")]
     NoKeyFound(String),
     /// The Key shortcut was formatted incorrectly (like "++" or "+control")
     ///
     /// Listing (`key_bind`)
+    #[error("Failed to parse Key because of a trailing delimiter in input: {0:#?}")]
     TrailingDelimiter(String),
     /// Error when multiple keys are found (like "Q+E")
     ///
     /// Listing (`key_bind`, (`old_key`, `new_key`))
-    MultipleKeys(String, (String, String)),
+    #[error("Failed to parse Key because multiple non-modifier keys were found, keys: [{old_key}, {new_key}], input: {input:#?}")]
+    MultipleKeys {
+        input: String,
+        old_key: String,
+        new_key: String,
+    },
     /// Error when a unknown value is found (a value that could not be parsed as a key or modifier)
     ///
     /// Example being a value that is not 1 length, starts with "f" and has numbers following or is a match against [`const_keys`].
     /// like `"    "`
     ///
     /// Listing (`key_bind`)
+    #[error("Failed to parse Key because of unknown key in mapping: {0:#?}")]
     UnknownKey(String),
 }
 
-impl Display for KeyParseError {
-    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
-        write!(
-            f,
-            "Failed to parse Key because {}",
-            // "{:#?}" debug representation is explicitly used here to escape the string contents
-            match self {
-                Self::NoKeyFound(val) =>
-                    format!("no key was found in the mapping, given: {val:#?}"),
-                Self::TrailingDelimiter(val) => format!("trailing delimiter in key: {val:#?}"),
-                Self::MultipleKeys(val, keys) => format!(
-                    "multiple keys were found, keys: [{}, {}], mapping: {:#?}",
-                    keys.0, keys.1, val
-                ),
-                Self::UnknownKey(val) => format!("of unknown key in mapping: {val:#?}"),
-            }
-        )
-    }
-}
-
-impl Error for KeyParseError {}
-
 // Note: this could likely be optimized / improved when the std patters becomes available (to match "".split('')), see https://github.com/rust-lang/rust/issues/27721
 /// A [`str::split`] replacement that works similar to `str::split(_, '+')`, but can also return the delimiter if directly followed
 /// like `"control++"` separates it into `["control", "+"]`.
@@ -1339,7 +1324,7 @@ impl<'a> Iterator for SplitAtPlus<'a> {
                         self.last_char_was_returned_delim = false;
                         self.last_char_was_delim = true;
                         continue;
-                    } else if i == 0 && self.chars.peek().map_or(false, |v| v.1 != Self::DELIM) {
+                    } else if i == 0 && self.chars.peek().is_some_and(|v| v.1 != Self::DELIM) {
                         // special case where the delimiter is the first, but not followed by another delimiter, like "+q"
                         // this is so we return a InvalidFormat later on (treat the first "+" as a delimiter instead of a key)
                         self.last_char_was_returned_delim = false;
@@ -1402,19 +1387,17 @@ impl KeyBinding {
         for val in SplitAtPlus::new(&input) {
             // make a trailing "+" as a error, like "q+"
             if val.is_empty() {
-                return Err(KeyParseError::TrailingDelimiter(input.clone()));
+                return Err(KeyParseError::TrailingDelimiter(input));
             }
 
             if let Ok(new_key) = KeyWrap::try_from(val) {
                 let opt: &mut Option<tuievents::Key> = &mut key_opt;
                 if let Some(existing_key) = opt {
-                    return Err(KeyParseError::MultipleKeys(
-                        input.clone(),
-                        (
-                            KeyWrap::from(*existing_key).to_string(),
-                            new_key.to_string(),
-                        ),
-                    ));
+                    return Err(KeyParseError::MultipleKeys {
+                        input,
+                        old_key: KeyWrap::from(*existing_key).to_string(),
+                        new_key: new_key.to_string(),
+                    });
                 }
 
                 *opt = Some(new_key.0);
@@ -1432,7 +1415,7 @@ impl KeyBinding {
         }
 
         let Some(mut code) = key_opt else {
-            return Err(KeyParseError::NoKeyFound(input.clone()));
+            return Err(KeyParseError::NoKeyFound(input));
         };
 
         // transform the key to be upper-case if "Shift" is enabled, as that is what tuirealm will provide (and we cannot modify that)
@@ -1449,12 +1432,14 @@ impl KeyBinding {
 
     /// Get the inner key
     #[inline]
+    #[must_use]
     pub fn get(&self) -> tuievents::KeyEvent {
         self.key_event
     }
 
     /// Get the Current Modifier, and the string representation of the key
     #[inline]
+    #[must_use]
     pub fn mod_key(&self) -> (tuievents::KeyModifiers, String) {
         (
             self.key_event.modifiers,
@@ -1725,6 +1710,22 @@ impl Display for KeyWrap {
 
             // i literally have no clue what key this is supposed to be
             tuievents::Key::KeypadBegin => unimplemented!(),
+
+            // the following are new events with tuirealm 2.0, but only available in backend "termion", which we dont use
+            tuievents::Key::ShiftLeft
+            | tuievents::Key::AltLeft
+            | tuievents::Key::CtrlLeft
+            | tuievents::Key::ShiftRight
+            | tuievents::Key::AltRight
+            | tuievents::Key::CtrlRight
+            | tuievents::Key::ShiftUp
+            | tuievents::Key::AltUp
+            | tuievents::Key::CtrlUp
+            | tuievents::Key::ShiftDown
+            | tuievents::Key::AltDown
+            | tuievents::Key::CtrlDown
+            | tuievents::Key::CtrlHome
+            | tuievents::Key::CtrlEnd => unimplemented!(),
         }
     }
 }
@@ -1847,7 +1848,7 @@ mod v1_interop {
             ) {
                 // the old impl had lowercase and uppercase characters, need to compare them equally
                 (tuievents::Key::Char(left), tuievents::Key::Char(right)) => {
-                    left.to_ascii_lowercase() == right.to_ascii_lowercase()
+                    left.eq_ignore_ascii_case(&right)
                 }
                 (left, right) => left == right,
             };
@@ -1984,6 +1985,7 @@ mod v1_interop {
         use pretty_assertions::assert_eq;
         use v1::BindingForEvent;
 
+        #[allow(clippy::too_many_lines)] // this test just requires a lot of fields
         #[test]
         fn should_convert_default_without_error() {
             let converted: Keys = v1::Keys::default().into();
@@ -2246,10 +2248,12 @@ mod v1_interop {
         #[test]
         fn should_fixup_old_volume_default() {
             let converted: Keys = {
-                let mut v1 = v1::Keys::default();
-                v1.global_player_volume_minus_2 = BindingForEvent {
-                    code: tuievents::Key::Char('_'),
-                    modifier: tuievents::KeyModifiers::SHIFT,
+                let v1 = v1::Keys {
+                    global_player_volume_minus_2: BindingForEvent {
+                        code: tuievents::Key::Char('_'),
+                        modifier: tuievents::KeyModifiers::SHIFT,
+                    },
+                    ..v1::Keys::default()
                 };
 
                 v1.into()
@@ -2488,10 +2492,11 @@ mod test {
         #[test]
         fn should_error_on_multiple_keys() {
             assert_eq!(
-                Err(KeyParseError::MultipleKeys(
-                    "q+s".to_owned(),
-                    ("q".to_owned(), "s".to_string())
-                )),
+                Err(KeyParseError::MultipleKeys {
+                    input: "q+s".to_string(),
+                    old_key: "q".to_string(),
+                    new_key: "s".to_string()
+                }),
                 KeyBinding::try_from("Q+S")
             );
         }
