@@ -1,6 +1,6 @@
 #![allow(clippy::module_name_repetitions)]
 
-use std::{error::Error, fmt::Display, fs::File, io::BufReader, num::ParseIntError, path::Path};
+use std::{fs::File, io::BufReader, num::ParseIntError, path::Path};
 
 use serde::{Deserialize, Serialize};
 use tuirealm::props::Color;
@@ -20,10 +20,14 @@ pub mod styles;
 #[serde(default)] // allow missing fields and fill them with the `..Self::default()` in this struct
 pub struct ThemeWrap {
     pub style: styles::Styles,
+    // On full-on default, also set the names to "Termusic Default"
+    // this function is only used if this property does not exist at all
+    #[serde(default = "ThemeColors::full_default")]
     pub theme: ThemeColors,
 }
 
 impl ThemeWrap {
+    #[must_use]
     pub fn get_color_from_theme(&self, color: ColorTermusic) -> Color {
         match color {
             ColorTermusic::Reset => Color::Reset,
@@ -49,140 +53,143 @@ impl ThemeWrap {
     }
 
     #[inline]
+    #[must_use]
     pub fn library_foreground(&self) -> Color {
         self.get_color_from_theme(self.style.library.foreground_color)
     }
 
     #[inline]
+    #[must_use]
     pub fn library_background(&self) -> Color {
         self.get_color_from_theme(self.style.library.background_color)
     }
 
     #[inline]
+    #[must_use]
     pub fn library_highlight(&self) -> Color {
         self.get_color_from_theme(self.style.library.highlight_color)
     }
 
     #[inline]
+    #[must_use]
     pub fn library_border(&self) -> Color {
         self.get_color_from_theme(self.style.library.border_color)
     }
 
     #[inline]
+    #[must_use]
     pub fn playlist_foreground(&self) -> Color {
         self.get_color_from_theme(self.style.playlist.foreground_color)
     }
 
     #[inline]
+    #[must_use]
     pub fn playlist_background(&self) -> Color {
         self.get_color_from_theme(self.style.playlist.background_color)
     }
 
     #[inline]
+    #[must_use]
     pub fn playlist_highlight(&self) -> Color {
         self.get_color_from_theme(self.style.playlist.highlight_color)
     }
 
     #[inline]
+    #[must_use]
     pub fn playlist_border(&self) -> Color {
         self.get_color_from_theme(self.style.playlist.border_color)
     }
 
     #[inline]
+    #[must_use]
     pub fn progress_foreground(&self) -> Color {
         self.get_color_from_theme(self.style.progress.foreground_color)
     }
 
     #[inline]
+    #[must_use]
     pub fn progress_background(&self) -> Color {
         self.get_color_from_theme(self.style.progress.background_color)
     }
 
     #[inline]
+    #[must_use]
     pub fn progress_border(&self) -> Color {
         self.get_color_from_theme(self.style.progress.border_color)
     }
 
     #[inline]
+    #[must_use]
     pub fn lyric_foreground(&self) -> Color {
         self.get_color_from_theme(self.style.lyric.foreground_color)
     }
 
     #[inline]
+    #[must_use]
     pub fn lyric_background(&self) -> Color {
         self.get_color_from_theme(self.style.lyric.background_color)
     }
 
     #[inline]
+    #[must_use]
     pub fn lyric_border(&self) -> Color {
         self.get_color_from_theme(self.style.lyric.border_color)
     }
 
     #[inline]
+    #[must_use]
     pub fn important_popup_foreground(&self) -> Color {
         self.get_color_from_theme(self.style.important_popup.foreground_color)
     }
 
     #[inline]
+    #[must_use]
     pub fn important_popup_background(&self) -> Color {
         self.get_color_from_theme(self.style.important_popup.background_color)
     }
 
     #[inline]
+    #[must_use]
     pub fn important_popup_border(&self) -> Color {
         self.get_color_from_theme(self.style.important_popup.border_color)
     }
 
     #[inline]
+    #[must_use]
     pub fn fallback_foreground(&self) -> Color {
         self.get_color_from_theme(self.style.fallback.foreground_color)
     }
 
     #[inline]
+    #[must_use]
     pub fn fallback_background(&self) -> Color {
         self.get_color_from_theme(self.style.fallback.background_color)
     }
 
     #[inline]
+    #[must_use]
     pub fn fallback_highlight(&self) -> Color {
         self.get_color_from_theme(self.style.fallback.highlight_color)
     }
 
     #[inline]
+    #[must_use]
     pub fn fallback_border(&self) -> Color {
         self.get_color_from_theme(self.style.fallback.border_color)
     }
 }
 
-// TODO: consider upgrading this with "thiserror"
 /// Error for when [`ThemeColor`] parsing fails
-#[derive(Debug, Clone, PartialEq)]
+#[derive(Debug, Clone, PartialEq, thiserror::Error)]
 pub enum ThemeColorParseError {
-    ParseIntError(ParseIntError),
+    #[error("Failed to parse color because of {0}")]
+    ParseIntError(#[from] ParseIntError),
+    #[error("Failed to parse color because of incorrect length {0}, expected prefix \"#\" or \"0x\" and length 6")]
     IncorrectLength(usize),
+    #[error("Failed to parse color becazse of unknown prefix \"{0}\", expected \"#\" or \"0x\"")]
+    UnknownPrefix(String),
 }
 
-impl Display for ThemeColorParseError {
-    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
-        let alternate = f.alternate();
-        write!(
-            f,
-            "Failed to parse color because of {}",
-            match self {
-                Self::ParseIntError(v) =>
-                    if alternate {
-                        format!("{v:#}")
-                    } else {
-                        format!("{v}")
-                    },
-                Self::IncorrectLength(length) => format!("Incorrect length {length}, expected 6"),
-            }
-        )
-    }
-}
-
-impl Error for ThemeColorParseError {}
-
 /// The rgb colors
 #[derive(Debug, Copy, Clone, Deserialize, Serialize, PartialEq, Eq)]
 #[serde(try_from = "String")]
@@ -194,9 +201,17 @@ pub struct ThemeColor {
 }
 
 impl ThemeColor {
+    /// Create a new instance with those values
+    #[must_use]
+    pub const fn new(r: u8, g: u8, b: u8) -> Self {
+        Self { r, g, b }
+    }
+
     /// Convert from a prefix + 6 length string
     pub fn from_hex(val: &str) -> Result<Self, ThemeColorParseError> {
-        let without_prefix = val.trim_start_matches('#');
+        let Some(without_prefix) = val.strip_prefix('#').or(val.strip_prefix("0x")) else {
+            return Err(ThemeColorParseError::UnknownPrefix(val.to_string()));
+        };
 
         // not in a format we support
         if without_prefix.len() != 6 {
@@ -215,6 +230,7 @@ impl ThemeColor {
 
     /// Convert to hex prefix + 6 length string
     #[inline]
+    #[must_use]
     pub fn to_hex(&self) -> String {
         format!("#{:02x}{:02x}{:02x}", self.r, self.g, self.b)
     }
@@ -261,6 +277,12 @@ impl From<ThemeColor> for Color {
 #[derive(Debug, Clone, Deserialize, Serialize, PartialEq, Eq)]
 #[serde(default)] // allow missing fields and fill them with the `..Self::default()` in this struct
 pub struct ThemeColors {
+    /// The Filename of the current theme, if a file is used.
+    /// This value is skipped if empty.
+    ///
+    /// This is used for example to pre-select in the config editor if available.
+    #[serde(skip_serializing_if = "Option::is_none")]
+    pub file_name: Option<String>,
     pub name: String,
     pub author: String,
     pub primary: ThemePrimary,
@@ -272,6 +294,7 @@ pub struct ThemeColors {
 impl Default for ThemeColors {
     fn default() -> Self {
         Self {
+            file_name: None,
             name: default_name(),
             author: default_author(),
             primary: ThemePrimary::default(),
@@ -282,37 +305,25 @@ impl Default for ThemeColors {
     }
 }
 
-// TODO: consider upgrading this with "thiserror"
+impl ThemeColors {
+    /// Get the full default theme, including names.
+    ///
+    /// This function is different from [`Self::default`] as the trait impl is also used for filling empty places
+    #[must_use]
+    pub fn full_default() -> Self {
+        Self {
+            name: "Termusic Default".to_string(),
+            author: "Termusic Developers".to_string(),
+            ..Default::default()
+        }
+    }
+}
+
 /// Error for when [`ThemeColors`] parsing fails
-#[derive(Debug, Clone, PartialEq)]
+#[derive(Debug, Clone, PartialEq, thiserror::Error)]
 pub enum ThemeColorsParseError {
-    ThemeColor(ThemeColorParseError),
-}
-
-impl Display for ThemeColorsParseError {
-    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
-        let alternate = f.alternate();
-        write!(
-            f,
-            "Failed to parse Theme because of {}",
-            match self {
-                Self::ThemeColor(v) =>
-                    if alternate {
-                        format!("{v:#}")
-                    } else {
-                        format!("{v}")
-                    },
-            }
-        )
-    }
-}
-
-impl Error for ThemeColorsParseError {}
-
-impl From<ThemeColorParseError> for ThemeColorsParseError {
-    fn from(value: ThemeColorParseError) -> Self {
-        Self::ThemeColor(value)
-    }
+    #[error("Failed to parse Theme: {0}")]
+    ThemeColor(#[from] ThemeColorParseError),
 }
 
 impl TryFrom<YAMLTheme> for ThemeColors {
@@ -321,8 +332,9 @@ impl TryFrom<YAMLTheme> for ThemeColors {
     fn try_from(value: YAMLTheme) -> Result<Self, Self::Error> {
         let colors = value.colors;
         Ok(Self {
-            name: colors.name,
-            author: colors.author,
+            file_name: None,
+            name: colors.name.unwrap_or_else(default_name),
+            author: colors.author.unwrap_or_else(default_author),
             primary: colors.primary.try_into()?,
             cursor: colors.cursor.try_into()?,
             normal: colors.normal.try_into()?,
@@ -336,7 +348,12 @@ impl ThemeColors {
     pub fn from_yaml_file(path: &Path) -> anyhow::Result<Self> {
         let parsed: YAMLTheme = serde_yaml::from_reader(BufReader::new(File::open(path)?))?;
 
-        Ok(Self::try_from(parsed)?)
+        let mut theme = Self::try_from(parsed)?;
+
+        let file_name = path.file_stem();
+        theme.file_name = file_name.map(|v| v.to_string_lossy().to_string());
+
+        Ok(theme)
     }
 }
 
@@ -349,8 +366,8 @@ pub struct ThemePrimary {
 impl Default for ThemePrimary {
     fn default() -> Self {
         Self {
-            background: default_000(),
-            foreground: default_fff(),
+            background: ThemeColor::new(0x10, 0x14, 0x21),
+            foreground: ThemeColor::new(0xff, 0xfb, 0xf6),
         }
     }
 }
@@ -376,7 +393,7 @@ pub struct ThemeCursor {
 impl Default for ThemeCursor {
     fn default() -> Self {
         Self {
-            text: default_fff(),
+            text: ThemeColor::new(0x1e, 0x1e, 0x1e),
             cursor: default_fff(),
         }
     }
@@ -409,14 +426,14 @@ pub struct ThemeNormal {
 impl Default for ThemeNormal {
     fn default() -> Self {
         Self {
-            black: default_000(),
-            red: ThemeColor::from_hex("#ff0000").unwrap(),
-            green: ThemeColor::from_hex("#00ff00").unwrap(),
-            yellow: ThemeColor::from_hex("#ffff00").unwrap(),
-            blue: ThemeColor::from_hex("#0000ff").unwrap(),
-            magenta: ThemeColor::from_hex("#ff00ff").unwrap(),
-            cyan: ThemeColor::from_hex("#00ffff").unwrap(),
-            white: default_fff(),
+            black: ThemeColor::new(0x2e, 0x2e, 0x2e),
+            red: ThemeColor::new(0xeb, 0x41, 0x29),
+            green: ThemeColor::new(0xab, 0xe0, 0x47),
+            yellow: ThemeColor::new(0xf6, 0xc7, 0x44),
+            blue: ThemeColor::new(0x47, 0xa0, 0xf3),
+            magenta: ThemeColor::new(0x7b, 0x5c, 0xb0),
+            cyan: ThemeColor::new(0x64, 0xdb, 0xed),
+            white: ThemeColor::new(0xe5, 0xe9, 0xf0),
         }
     }
 }
@@ -454,14 +471,14 @@ pub struct ThemeBright {
 impl Default for ThemeBright {
     fn default() -> Self {
         Self {
-            black: ThemeColor::from_hex("#777777").unwrap(),
-            red: default_000(),
-            green: default_000(),
-            yellow: default_000(),
-            blue: default_000(),
-            magenta: default_000(),
-            cyan: default_000(),
-            white: default_000(),
+            black: ThemeColor::new(0x56, 0x56, 0x56),
+            red: ThemeColor::new(0xec, 0x53, 0x57),
+            green: ThemeColor::new(0xc0, 0xe1, 0x7d),
+            yellow: ThemeColor::new(0xf9, 0xda, 0x6a),
+            blue: ThemeColor::new(0x49, 0xa4, 0xf8),
+            magenta: ThemeColor::new(0xa4, 0x7d, 0xe9),
+            cyan: ThemeColor::new(0x99, 0xfa, 0xf2),
+            white: default_fff(),
         }
     }
 }
@@ -493,14 +510,9 @@ fn default_author() -> String {
     "empty author".to_string()
 }
 
-#[inline]
-fn default_000() -> ThemeColor {
-    ThemeColor::from_hex("#000000").unwrap()
-}
-
 #[inline]
 fn default_fff() -> ThemeColor {
-    ThemeColor::from_hex("#FFFFFF").unwrap()
+    ThemeColor::new(0xFF, 0xFF, 0xFF)
 }
 
 mod v1_interop {
@@ -510,6 +522,7 @@ mod v1_interop {
     impl From<&v1::Alacritty> for ThemeColors {
         fn from(value: &v1::Alacritty) -> Self {
             Self {
+                file_name: None,
                 name: value.name.clone(),
                 author: value.author.clone(),
                 primary: ThemePrimary {
@@ -564,6 +577,7 @@ mod v1_interop {
             assert_eq!(
                 converted,
                 ThemeColors {
+                    file_name: None,
                     name: "default".into(),
                     author: "Larry Hao".into(),
                     primary: ThemePrimary {
@@ -599,3 +613,50 @@ mod v1_interop {
         }
     }
 }
+
+#[cfg(test)]
+mod tests {
+    use super::ThemeColors;
+
+    mod theme_color {
+        use super::super::ThemeColor;
+
+        #[test]
+        fn should_parse_hashtag() {
+            assert_eq!(
+                ThemeColor::new(1, 2, 3),
+                ThemeColor::from_hex("#010203").unwrap()
+            );
+            assert_eq!(
+                ThemeColor::new(255, 255, 255),
+                ThemeColor::from_hex("#ffffff").unwrap()
+            );
+            assert_eq!(
+                ThemeColor::new(0, 0, 0),
+                ThemeColor::from_hex("#000000").unwrap()
+            );
+        }
+
+        #[test]
+        fn should_parse_0x() {
+            assert_eq!(
+                ThemeColor::new(1, 2, 3),
+                ThemeColor::from_hex("0x010203").unwrap()
+            );
+            assert_eq!(
+                ThemeColor::new(255, 255, 255),
+                ThemeColor::from_hex("0xffffff").unwrap()
+            );
+            assert_eq!(
+                ThemeColor::new(0, 0, 0),
+                ThemeColor::from_hex("0x000000").unwrap()
+            );
+        }
+    }
+
+    #[test]
+    fn should_default() {
+        // Test that there are no panics in the defaults, this should be able to be omitted once it is const
+        let _ = ThemeColors::default();
+    }
+}
