@@ -1,3 +1,4 @@
+use crate::library_db::const_unknown::{UNKNOWN_ARTIST, UNKNOWN_TITLE};
 use crate::podcast::episode::Episode;
 /**
  * MIT License
@@ -25,45 +26,52 @@ use crate::podcast::episode::Episode;
 use crate::songtag::lrc::Lyric;
 use crate::utils::get_parent_folder;
 use anyhow::{bail, Context, Result};
-use id3::frame::Lyrics;
-use lofty::config::{ParseOptions, WriteOptions};
+use id3::frame::Lyrics as Id3Lyrics;
+use lofty::config::WriteOptions;
 use lofty::id3::v2::{Frame, Id3v2Tag, UnsynchronizedTextFrame};
 use lofty::picture::{Picture, PictureType};
 use lofty::prelude::{Accessor, AudioFile, ItemKey, TagExt, TaggedFileExt};
 use lofty::tag::{ItemValue, Tag as LoftyTag, TagItem};
-use lofty::{file::FileType, mpeg::MpegFile, probe::Probe, TextEncoding};
+use lofty::{file::FileType, probe::Probe};
 use std::convert::From;
 use std::ffi::OsStr;
 use std::fs::rename;
-use std::fs::File;
-use std::io::BufReader;
-use std::path::Path;
+use std::path::{Path, PathBuf};
 use std::str::FromStr;
 use std::time::{Duration, SystemTime};
 
-// TODO: add some kind of identifier for easy printing, like a uri that is NOT optional
+/// Location types for a Track, could be a local file with [`LocationType::Path`] or a remote URI with [`LocationType::Uri`]
+#[derive(Clone, Debug, PartialEq)]
+pub enum LocationType {
+    /// A Local file, for use with [`MediaType::Music`]
+    Path(PathBuf),
+    /// A remote URI, for use with [`MediaType::LiveRadio`] and [`MediaType::Podcast`]
+    Uri(String),
+}
+
+impl From<PathBuf> for LocationType {
+    fn from(value: PathBuf) -> Self {
+        Self::Path(value)
+    }
+}
+
 #[derive(Clone, Debug)]
 pub struct Track {
+    /// The URI or the Path of the file
+    location: LocationType,
+    pub media_type: MediaType,
+
     /// Artist of the song
     artist: Option<String>,
     /// Album of the song
     album: Option<String>,
     /// Title of the song
     title: Option<String>,
-    /// File path to the song
-    // TODO: why is this a http(s) url in Radio case?
-    file: Option<String>,
     /// Duration of the song
     duration: Duration,
-    /// Name of the song
-    name: Option<String>,
-    /// Extension of the song
-    ext: Option<String>,
-    // TODO: why is this set to the http(s) url in Radio case?
-    directory: Option<String>,
     pub last_modified: SystemTime,
     /// USLT lyrics
-    lyric_frames: Vec<Lyrics>,
+    lyric_frames: Vec<Id3Lyrics>,
     lyric_selected_index: usize,
     parsed_lyric: Option<Lyric>,
     picture: Option<Picture>,
@@ -76,13 +84,12 @@ pub struct Track {
     // Performer
     // Disc
     // Comment
-    pub media_type: MediaType,
     pub podcast_localfile: Option<String>,
 }
 
 impl PartialEq for Track {
     fn eq(&self, other: &Self) -> bool {
-        self.file == other.file
+        self.location == other.location
     }
 }
 
@@ -94,9 +101,11 @@ pub enum MediaType {
 }
 
 impl Track {
+    /// Create a new [`MediaType::Podcast`] track
     #[allow(clippy::cast_sign_loss)]
+    #[must_use]
     pub fn from_episode(ep: &Episode) -> Self {
-        let lyric_frames: Vec<Lyrics> = Vec::new();
+        let lyric_frames: Vec<Id3Lyrics> = Vec::new();
         let mut podcast_localfile: Option<String> = None;
         if let Some(path) = &ep.path {
             if path.exists() {
@@ -107,13 +116,9 @@ impl Track {
         Self {
             artist: Some("Episode".to_string()),
             album: None,
-            // album: Some(ep.description.clone()),
             title: Some(ep.title.clone()),
-            file: Some(ep.url.clone()),
+            location: LocationType::Uri(ep.url.clone()),
             duration: Duration::from_secs(ep.duration.unwrap_or(0) as u64),
-            name: None,
-            ext: None,
-            directory: None,
             last_modified: SystemTime::now(),
             lyric_frames,
             lyric_selected_index: 0,
@@ -127,83 +132,46 @@ impl Track {
         }
     }
 
+    /// Create a new [`MediaType::Music`] track
     pub fn read_from_path<P: AsRef<Path>>(path: P, for_db: bool) -> Result<Self> {
         let path = path.as_ref();
 
         let probe = Probe::open(path)?;
-        let file_type = probe.file_type();
 
-        let mut song = Self::new(path);
-        if let Ok(mut tagged_file) = probe.read() {
+        let mut song = Self::new(LocationType::Path(path.to_path_buf()), MediaType::Music);
+        let tagged_file = match probe.read() {
+            Ok(v) => Some(v),
+            Err(err) => {
+                warn!(
+                    "Failed to read metadata from \"{}\": {}",
+                    path.display(),
+                    err
+                );
+                None
+            }
+        };
+
+        if let Some(mut tagged_file) = tagged_file {
             // We can at most get the duration and file type at this point
             let properties = tagged_file.properties();
             song.duration = properties.duration();
             song.file_type = Some(tagged_file.file_type());
 
             if let Some(tag) = tagged_file.primary_tag_mut() {
-                // Check for a length tag (Ex. TLEN in ID3v2)
-                if let Some(len_tag) = tag.get_string(&ItemKey::Length) {
-                    song.duration = Duration::from_millis(len_tag.parse::<u64>()?);
-                }
-
-                song.artist = tag.artist().map(std::borrow::Cow::into_owned);
-                song.album = tag.album().map(std::borrow::Cow::into_owned);
-                song.title = tag.title().map(std::borrow::Cow::into_owned);
-                song.genre = tag.genre().map(std::borrow::Cow::into_owned);
-                song.media_type = MediaType::Music;
-
-                if for_db {
-                    return Ok(song);
-                }
-
-                // Get all of the lyrics tags
-                let mut lyric_frames: Vec<Lyrics> = Vec::new();
-                match file_type {
-                    Some(FileType::Mpeg) => {
-                        let mut reader = BufReader::new(File::open(path)?);
-                        // let file = MPEGFile::read_from(&mut reader, false)?;
-                        let file = MpegFile::read_from(&mut reader, ParseOptions::new())?;
-
-                        if let Some(id3v2_tag) = file.id3v2() {
-                            for lyrics_frame in id3v2_tag.unsync_text() {
-                                let mut language =
-                                    String::from_utf8_lossy(&lyrics_frame.language).to_string();
-                                if language.len() < 3 {
-                                    language = "eng".to_string();
-                                }
-                                lyric_frames.push(Lyrics {
-                                    lang: language,
-                                    description: lyrics_frame.description.clone(),
-                                    text: lyrics_frame.content.clone(),
-                                });
-                            }
-                        }
-                    }
-                    _ => {
-                        create_lyrics(tag, &mut lyric_frames);
-                    }
-                };
-                song.parsed_lyric = lyric_frames
-                    .first()
-                    .map(|lf| Lyric::from_str(&lf.text).ok())
-                    .and_then(|pl| pl);
-                song.lyric_frames = lyric_frames;
-
-                // Get the picture (not necessarily the front cover)
-                let mut picture = tag
-                    .pictures()
-                    .iter()
-                    .find(|pic| pic.pic_type() == PictureType::CoverFront)
-                    .cloned();
-                if picture.is_none() {
-                    picture = tag.pictures().first().cloned();
-                }
-
-                song.picture = picture;
+                Self::process_tag(tag, &mut song, for_db)?;
+            } else if let Some(tag) = tagged_file.first_tag_mut() {
+                Self::process_tag(tag, &mut song, for_db)?;
+            } else {
+                warn!("File \"{}\" does not have any tags!", path.display());
             }
         }
 
-        let parent_folder = get_parent_folder(&path.to_string_lossy());
+        // exit early if its for db only as no cover is needed there
+        if for_db {
+            return Ok(song);
+        }
+
+        let parent_folder = get_parent_folder(path);
 
         if let Ok(files) = std::fs::read_dir(parent_folder) {
             for f in files.flatten() {
@@ -219,56 +187,87 @@ impl Track {
         Ok(song)
     }
 
+    /// Process a given [`LoftyTag`] into the given `track`
+    fn process_tag(tag: &mut LoftyTag, track: &mut Track, for_db: bool) -> Result<()> {
+        // Check for a length tag (Ex. TLEN in ID3v2)
+        if let Some(len_tag) = tag.get_string(&ItemKey::Length) {
+            track.duration = Duration::from_millis(len_tag.parse::<u64>()?);
+        }
+
+        track.artist = tag.artist().map(std::borrow::Cow::into_owned);
+        track.album = tag.album().map(std::borrow::Cow::into_owned);
+        track.title = tag.title().map(std::borrow::Cow::into_owned);
+        track.genre = tag.genre().map(std::borrow::Cow::into_owned);
+        track.media_type = MediaType::Music;
+
+        if for_db {
+            return Ok(());
+        }
+
+        // Get all of the lyrics tags
+        let mut lyric_frames: Vec<Id3Lyrics> = Vec::new();
+        create_lyrics(tag, &mut lyric_frames);
+
+        track.parsed_lyric = lyric_frames
+            .first()
+            .and_then(|lf| Lyric::from_str(&lf.text).ok());
+        track.lyric_frames = lyric_frames;
+
+        // Get the picture (not necessarily the front cover)
+        let picture = tag
+            .pictures()
+            .iter()
+            .find(|pic| pic.pic_type() == PictureType::CoverFront)
+            .or_else(|| tag.pictures().first())
+            .cloned();
+
+        track.picture = picture;
+
+        Ok(())
+    }
+
+    /// Create a new [`MediaType::LiveRadio`] track
+    #[must_use]
     pub fn new_radio(url: &str) -> Self {
-        let mut track = Self::new(url);
+        let mut track = Self::new(LocationType::Uri(url.to_string()), MediaType::LiveRadio);
         track.artist = Some("Radio".to_string());
         track.title = Some("Radio Station".to_string());
         track.album = Some("Live".to_string());
-        track.media_type = MediaType::LiveRadio;
         track
     }
-    fn new<P: AsRef<Path>>(path: P) -> Self {
-        let p = path.as_ref();
-        let directory = Some(get_parent_folder(&p.to_string_lossy()));
-        let ext = p.extension().and_then(OsStr::to_str).map(String::from);
-        let artist = Some(String::from("Unsupported?"));
-        let album = Some(String::from("Unsupported?"));
-        let title = p.file_stem().and_then(OsStr::to_str).map(String::from);
-        let file = Some(p.to_string_lossy().into_owned());
+
+    #[must_use]
+    fn new(location: LocationType, media_type: MediaType) -> Self {
         let duration = Duration::from_secs(0);
-        let name = p
-            .file_name()
-            .and_then(OsStr::to_str)
-            .map(std::string::ToString::to_string);
-        let parsed_lyric: Option<Lyric> = None;
-        let lyric_frames: Vec<Lyrics> = Vec::new();
-        let picture: Option<Picture> = None;
-        let album_photo: Option<String> = None;
-        let genre = Some(String::from("Unknown"));
+        let lyric_frames: Vec<Id3Lyrics> = Vec::new();
         let mut last_modified = SystemTime::now();
-        if let Ok(meta) = p.metadata() {
-            if let Ok(modified) = meta.modified() {
-                last_modified = modified;
+        let mut title = None;
+
+        if let LocationType::Path(path) = &location {
+            if let Ok(meta) = path.metadata() {
+                if let Ok(modified) = meta.modified() {
+                    last_modified = modified;
+                }
             }
+
+            title = path.file_stem().and_then(OsStr::to_str).map(String::from);
         }
+
         Self {
-            ext,
             file_type: None,
-            artist,
-            album,
+            artist: None,
+            album: None,
             title,
-            file,
-            directory,
             duration,
-            name,
-            parsed_lyric,
+            location,
+            parsed_lyric: None,
             lyric_frames,
             lyric_selected_index: 0,
-            picture,
-            album_photo,
+            picture: None,
+            album_photo: None,
             last_modified,
-            genre,
-            media_type: MediaType::Music,
+            genre: None,
+            media_type,
             podcast_localfile: None,
         }
     }
@@ -283,7 +282,7 @@ impl Track {
         Ok(())
     }
 
-    pub fn cycle_lyrics(&mut self) -> Result<&Lyrics> {
+    pub fn cycle_lyrics(&mut self) -> Result<&Id3Lyrics> {
         if self.lyric_frames_is_empty() {
             bail!("no lyrics embedded");
         }
@@ -303,6 +302,7 @@ impl Track {
         bail!("cycle lyrics error")
     }
 
+    #[must_use]
     pub const fn parsed_lyric(&self) -> Option<&Lyric> {
         self.parsed_lyric.as_ref()
     }
@@ -319,11 +319,13 @@ impl Track {
         self.lyric_selected_index = index;
     }
 
+    #[must_use]
     pub const fn lyric_selected_index(&self) -> usize {
         self.lyric_selected_index
     }
 
-    pub fn lyric_selected(&self) -> Option<&Lyrics> {
+    #[must_use]
+    pub fn lyric_selected(&self) -> Option<&Id3Lyrics> {
         if self.lyric_frames.is_empty() {
             return None;
         }
@@ -333,10 +335,12 @@ impl Track {
         None
     }
 
+    #[must_use]
     pub fn lyric_frames_is_empty(&self) -> bool {
         self.lyric_frames.is_empty()
     }
 
+    #[must_use]
     pub fn lyric_frames_len(&self) -> usize {
         if self.lyric_frames.is_empty() {
             return 0;
@@ -344,22 +348,26 @@ impl Track {
         self.lyric_frames.len()
     }
 
-    pub fn lyric_frames(&self) -> Option<Vec<Lyrics>> {
+    #[must_use]
+    pub fn lyric_frames(&self) -> Option<Vec<Id3Lyrics>> {
         if self.lyric_frames.is_empty() {
             return None;
         }
         Some(self.lyric_frames.clone())
     }
 
+    #[must_use]
     pub const fn picture(&self) -> Option<&Picture> {
         self.picture.as_ref()
     }
+    #[must_use]
     pub fn album_photo(&self) -> Option<&str> {
         self.album_photo.as_deref()
     }
 
     /// Optionally return the artist of the song
     /// If `None` it wasn't able to read the tags
+    #[must_use]
     pub fn artist(&self) -> Option<&str> {
         self.artist.as_deref()
     }
@@ -370,6 +378,7 @@ impl Track {
 
     /// Optionally return the song's album
     /// If `None` failed to read the tags
+    #[must_use]
     pub fn album(&self) -> Option<&str> {
         self.album.as_deref()
     }
@@ -378,6 +387,7 @@ impl Track {
         self.album = Some(album.to_string());
     }
 
+    #[must_use]
     pub fn genre(&self) -> Option<&str> {
         self.genre.as_deref()
     }
@@ -389,6 +399,7 @@ impl Track {
 
     /// Optionally return the title of the song
     /// If `None` it wasn't able to read the tags
+    #[must_use]
     pub fn title(&self) -> Option<&str> {
         self.title.as_deref()
     }
@@ -397,26 +408,45 @@ impl Track {
         self.title = Some(title.to_string());
     }
 
+    /// Get the full Path or URI of the track, if its a local file
+    #[must_use]
     pub fn file(&self) -> Option<&str> {
-        self.file.as_deref()
+        match &self.location {
+            LocationType::Path(path_buf) => path_buf.to_str(),
+            LocationType::Uri(uri) => Some(uri),
+        }
     }
 
+    /// Get the directory the track is in, if its a local file
     pub fn directory(&self) -> Option<&str> {
-        self.directory.as_deref()
+        if let LocationType::Path(path) = &self.location {
+            // not using "utils::get_parent_directory" as if a track is "LocationType::Path", it should have a directory and a file in the path
+            path.parent().and_then(Path::to_str)
+        } else {
+            None
+        }
     }
 
+    /// Get the extension of the track, if its a local file
     pub fn ext(&self) -> Option<&str> {
-        self.ext.as_deref()
+        if let LocationType::Path(path) = &self.location {
+            path.extension().and_then(OsStr::to_str)
+        } else {
+            None
+        }
     }
 
+    #[must_use]
     pub const fn duration(&self) -> Duration {
         self.duration
     }
 
+    #[must_use]
     pub fn duration_formatted(&self) -> String {
         Self::duration_formatted_short(&self.duration)
     }
 
+    #[must_use]
     pub fn duration_formatted_short(d: &Duration) -> String {
         let duration_hour = d.as_secs() / 3600;
         let duration_min = (d.as_secs() % 3600) / 60;
@@ -429,8 +459,13 @@ impl Track {
         }
     }
 
+    /// Get the `file_name` or the full URI of the current Track
     pub fn name(&self) -> Option<&str> {
-        self.name.as_deref()
+        match &self.location {
+            LocationType::Path(path) => path.file_name().and_then(OsStr::to_str),
+            // TODO: should this really return the uri here instead of None?
+            LocationType::Uri(uri) => Some(uri),
+        }
     }
 
     pub fn save_tag(&mut self) -> Result<()> {
@@ -445,13 +480,14 @@ impl Track {
                             for l in lyric_frames {
                                 let l_frame =
                                     Frame::UnsynchronizedText(UnsynchronizedTextFrame::new(
-                                        TextEncoding::UTF8,
+                                        lofty::TextEncoding::UTF8,
                                         l.lang.as_bytes()[0..3]
                                             .try_into()
                                             .with_context(|| "wrong length of language")?,
                                         l.description,
                                         l.text,
                                     ));
+
                                 tag.insert(l_frame);
                             }
                         }
@@ -464,7 +500,6 @@ impl Track {
                     tag.save_to_path(file_path, WriteOptions::new())?;
                 }
             }
-
             _ => {
                 if let Some(file_path) = self.file() {
                     let tag_type = match self.file_type {
@@ -500,8 +535,8 @@ impl Track {
         if let Some(ext) = self.ext() {
             let new_name = format!(
                 "{}-{}.{}",
-                self.artist().unwrap_or("Unknown Artist"),
-                self.title().unwrap_or("Unknown Title"),
+                self.artist().unwrap_or(UNKNOWN_ARTIST),
+                self.title().unwrap_or(UNKNOWN_TITLE),
                 ext,
             );
 
@@ -511,7 +546,7 @@ impl Track {
                 if let Some(p_prefix) = p_old.parent() {
                     let p_new = p_prefix.join(new_name_path);
                     rename(p_old, &p_new)?;
-                    self.file = Some(String::from(p_new.to_string_lossy()));
+                    self.location = LocationType::Path(p_new);
                 }
             }
         }
@@ -523,17 +558,14 @@ impl Track {
         let mut lyric_frames = self.lyric_frames.clone();
         match self.lyric_frames.get(self.lyric_selected_index) {
             Some(lyric_frame) => {
-                lyric_frames.remove(self.lyric_selected_index);
-                lyric_frames.insert(
-                    self.lyric_selected_index,
-                    Lyrics {
-                        text: lyric_str.to_string(),
-                        ..lyric_frame.clone()
-                    },
-                );
+                // No panic as the vec has just been cloned and using the same index into both vecs which has been checked
+                lyric_frames[self.lyric_selected_index] = Id3Lyrics {
+                    text: lyric_str.to_string(),
+                    ..lyric_frame.clone()
+                };
             }
             None => {
-                lyric_frames.push(Lyrics {
+                lyric_frames.push(Id3Lyrics {
                     lang: "eng".to_string(),
                     description: lang_ext.to_string(),
                     text: lyric_str.to_string(),
@@ -550,12 +582,12 @@ impl Track {
     fn update_tag<T: Accessor>(&self, tag: &mut T) {
         tag.set_artist(
             self.artist()
-                .map_or_else(|| String::from("Unknown Artist"), str::to_string),
+                .map_or_else(|| String::from(UNKNOWN_ARTIST), str::to_string),
         );
 
         tag.set_title(
             self.title()
-                .map_or_else(|| String::from("Unknown Title"), str::to_string),
+                .map_or_else(|| String::from(UNKNOWN_TITLE), str::to_string),
         );
 
         tag.set_album(self.album().map_or_else(String::new, str::to_string));
@@ -563,16 +595,20 @@ impl Track {
     }
 }
 
-fn create_lyrics(tag: &mut LoftyTag, lyric_frames: &mut Vec<Lyrics>) {
+fn create_lyrics(tag: &mut LoftyTag, lyric_frames: &mut Vec<Id3Lyrics>) {
     let lyrics = tag.take(&ItemKey::Lyrics);
     for lyric in lyrics {
         if let ItemValue::Text(lyrics_text) = lyric.value() {
-            lyric_frames.push(Lyrics {
-                lang: "eng".to_string(),
-                // description: String::new(),
-                description: "default".to_string(),
+            lyric_frames.push(Id3Lyrics {
+                lang: lyric.lang().escape_ascii().to_string(),
+                description: lyric.description().to_string(),
                 text: lyrics_text.to_string(),
             });
+            lyric_frames.sort_by(|a, b| {
+                a.description
+                    .to_lowercase()
+                    .cmp(&b.description.to_lowercase())
+            });
         }
     }
 }
