@@ -1,3 +1,5 @@
+use std::sync::Arc;
+
 use crate::config::v2::tui::{keys::KeyBinding, theme::styles::ColorTermusic};
 use crate::invidious::{Instance, YoutubeVideo};
 use crate::podcast::{EpData, PodcastFeed, PodcastNoId};
@@ -48,12 +50,19 @@ pub enum Msg {
     UpdatePhoto,
     YoutubeSearch(YSMsg),
     Xywh(XYWHMsg),
-    None,
+
+    /// Force a redraw because of some change.
+    ///
+    /// This is necessary as `Components` do not have access to `Model.redraw`.
+    ///
+    /// For example pushing ARROW DOWN to change the selection in a table.
+    ForceRedraw,
 }
 
 #[derive(Clone, Copy, PartialEq, Eq, Debug)]
 pub enum XYWHMsg {
-    Hide,
+    /// Toggle the hidden / shown status of the displayed image.
+    ToggleHidden,
     MoveLeft,
     MoveRight,
     MoveUp,
@@ -62,18 +71,47 @@ pub enum XYWHMsg {
     ZoomOut,
 }
 
+pub type DLMsgURL = Arc<str>;
+
 #[derive(Clone, PartialEq, Eq, Debug)]
 pub enum DLMsg {
-    DownloadRunning(String, String), // indicates progress
-    DownloadSuccess(String),
-    DownloadCompleted(String, Option<String>),
-    DownloadErrDownload(String, String, String),
-    DownloadErrEmbedData(String, String),
+    /// Indicates a Start of a download.
+    ///
+    /// `(Url, Title)`
+    DownloadRunning(DLMsgURL, String),
+    /// Indicates the Download was a Success, though termusic post-processing is not done yet.
+    ///
+    /// `(Url)`
+    DownloadSuccess(DLMsgURL),
+    /// Indicates the Download thread finished in both Success or Error.
+    ///
+    /// `(Url, Filename)`
+    DownloadCompleted(DLMsgURL, Option<String>),
+    /// Indicates that the Download has Errored and has been aborted.
+    ///
+    /// `(Url, Title, ErrorAsString)`
+    DownloadErrDownload(DLMsgURL, String, String),
+    /// Indicates that the Download was a Success, but termusic post-processing failed.
+    /// Like re-saving tags after editing.
+    ///
+    /// `(Url, Title)`
+    DownloadErrEmbedData(DLMsgURL, String),
+
+    // TODO: The Following 2 things have absolutely nothing to-do with Download
+    /// Show a status message in the TUI.
+    ///
+    /// `((Title, Text))`
     MessageShow((String, String)),
+    /// Hide a status message in the TUI.
+    ///
+    /// `((Title, Text))`
     MessageHide((String, String)),
-    YoutubeSearchSuccess(YoutubeOptions),
-    YoutubeSearchFail(String),
+
+    // TODO: The Following 2 things have absolutely nothing to-do with Download
+    /// Fetching & loading the image was a success, with the image.
     FetchPhotoSuccess(ImageWrapper),
+    /// Fetching & loading the image has failed, with error message.
+    /// `(ErrorAsString)`
     FetchPhotoErr(String),
 }
 
@@ -360,7 +398,7 @@ pub enum PCMsg {
     PodcastAddPopupCloseCancel,
     SyncData((i64, PodcastNoId)),
     NewData(PodcastNoId),
-    Error(String, PodcastFeed),
+    Error(PodcastFeed),
     PodcastSelected(usize),
     DescriptionUpdate,
     EpisodeAdd(usize),
@@ -454,6 +492,13 @@ pub enum YSMsg {
     TablePopupPrevious,
     TablePopupCloseCancel,
     TablePopupCloseOk(usize),
+
+    /// The youtube search was a success, with all values.
+    YoutubeSearchSuccess(YoutubeOptions),
+    /// Indicates that the youtube search has failed, with error message.
+    ///
+    /// `(ErrorAsString)`
+    YoutubeSearchFail(String),
 }
 #[derive(Clone, Debug, PartialEq, Eq)]
 pub enum TEMsg {
@@ -716,6 +761,7 @@ impl YoutubeOptions {
         Ok(())
     }
 
+    #[must_use]
     pub const fn page(&self) -> u32 {
         self.page
     }
