@@ -22,37 +22,10 @@
  * SOFTWARE.
  */
 
-// #![forbid(unsafe_code)]
-#![recursion_limit = "2048"]
-#![warn(clippy::all, clippy::correctness)]
-#![warn(rust_2018_idioms)]
-#![warn(clippy::pedantic)]
-#[allow(clippy::pedantic)]
-pub mod player {
-    tonic::include_proto!("player");
-
-    // implement transform function for easy use
-    impl From<Duration> for std::time::Duration {
-        fn from(value: Duration) -> Self {
-            std::time::Duration::new(value.secs, value.nanos)
-        }
-    }
-
-    impl From<std::time::Duration> for Duration {
-        fn from(value: std::time::Duration) -> Self {
-            Self {
-                secs: value.as_secs(),
-                nanos: value.subsec_nanos(),
-            }
-        }
-    }
-}
-
 #[cfg(feature = "gst")]
 mod gstreamer_backend;
 #[cfg(feature = "mpv")]
 mod mpv_backend;
-#[cfg(feature = "rusty")]
 mod rusty_backend;
 
 mod discord;
@@ -62,16 +35,18 @@ pub mod playlist;
 use anyhow::{Context, Result};
 use async_trait::async_trait;
 pub use playlist::{Playlist, Status};
-use serde::{Deserialize, Serialize};
 use std::time::Duration;
 use termusiclib::config::v2::server::config_extra::ServerConfigVersionedDefaulted;
-use termusiclib::config::{new_shared_server_settings, ServerOverlay, SharedServerSettings};
+use termusiclib::config::{ServerOverlay, SharedServerSettings};
 use termusiclib::library_db::DataBase;
+use termusiclib::player::{PlayerProgress, PlayerTimeUnit, TrackChangedInfo, UpdateEvents};
 use termusiclib::podcast::db::Database as DBPod;
 use termusiclib::track::{MediaType, Track};
 use termusiclib::utils::get_app_config_path;
 use tokio::runtime::Handle;
+use tokio::sync::mpsc::error::SendError;
 use tokio::sync::mpsc::{UnboundedReceiver, UnboundedSender};
+use tokio::sync::{broadcast, oneshot};
 
 #[macro_use]
 extern crate log;
@@ -80,19 +55,16 @@ extern crate log;
 pub enum BackendSelect {
     #[cfg(feature = "mpv")]
     Mpv,
-    #[cfg(feature = "rusty")]
-    Rusty,
     #[cfg(feature = "gst")]
     GStreamer,
     /// Create a new Backend with default backend ordering
     ///
     /// Order:
-    /// - [`RustyBackend`](rusty_backend::RustyBackend) (feature `rusty`)
     /// - [`GstreamerBackend`](gstreamer_backend::GStreamerBackend) (feature `gst`)
     /// - [`MpvBackend`](mpv_backend::MpvBackend) (feature `mpv`)
-    /// - Compile Error
+    /// - [`RustyBackend`](rusty_backend::RustyBackend) (default)
     #[default]
-    Default,
+    Rusty,
 }
 
 /// Enum to choose backend at runtime
@@ -100,14 +72,62 @@ pub enum BackendSelect {
 pub enum Backend {
     #[cfg(feature = "mpv")]
     Mpv(mpv_backend::MpvBackend),
-    #[cfg(feature = "rusty")]
     Rusty(rusty_backend::RustyBackend),
     #[cfg(feature = "gst")]
     GStreamer(gstreamer_backend::GStreamerBackend),
 }
 
-pub type PlayerCmdReciever = UnboundedReceiver<PlayerCmd>;
-pub type PlayerCmdSender = UnboundedSender<PlayerCmd>;
+pub type PlayerCmdCallback = oneshot::Receiver<()>;
+pub type PlayerCmdReciever = UnboundedReceiver<(PlayerCmd, PlayerCmdCallbackSender)>;
+
+/// Wrapper around the potential oneshot sender to implement convenience functions.
+#[derive(Debug)]
+pub struct PlayerCmdCallbackSender(Option<oneshot::Sender<()>>);
+
+impl PlayerCmdCallbackSender {
+    /// Send on the oneshot, if there is any.
+    pub fn call(self) {
+        let Some(sender) = self.0 else {
+            return;
+        };
+        let _ = sender.send(());
+    }
+}
+
+/// Wrapper for the actual sender, to make it easier to implement new functions.
+#[derive(Debug, Clone)]
+pub struct PlayerCmdSender(UnboundedSender<(PlayerCmd, PlayerCmdCallbackSender)>);
+
+impl PlayerCmdSender {
+    /// Send a given [`PlayerCmd`] without any callback.
+    ///
+    /// # Errors
+    /// Also see [`oneshot::Sender::send`].
+    pub fn send(
+        &self,
+        cmd: PlayerCmd,
+    ) -> Result<(), SendError<(PlayerCmd, PlayerCmdCallbackSender)>> {
+        self.0.send((cmd, PlayerCmdCallbackSender(None)))
+    }
+
+    /// Send a given [`PlayerCmd`] with a callback, returning the receiver.
+    ///
+    /// # Errors
+    /// Also see [`oneshot::Sender::send`].
+    pub fn send_cb(
+        &self,
+        cmd: PlayerCmd,
+    ) -> Result<PlayerCmdCallback, SendError<(PlayerCmd, PlayerCmdCallbackSender)>> {
+        let (tx, rx) = oneshot::channel();
+        self.0.send((cmd, PlayerCmdCallbackSender(Some(tx))))?;
+        Ok(rx)
+    }
+
+    #[must_use]
+    pub fn new(tx: UnboundedSender<(PlayerCmd, PlayerCmdCallbackSender)>) -> Self {
+        Self(tx)
+    }
+}
 
 impl Backend {
     /// Create a new Backend based on `backend`([`BackendSelect`])
@@ -115,32 +135,25 @@ impl Backend {
         match backend {
             #[cfg(feature = "mpv")]
             BackendSelect::Mpv => Self::new_mpv(config, cmd_tx),
-            #[cfg(feature = "rusty")]
-            BackendSelect::Rusty => Self::new_rusty(config, cmd_tx),
             #[cfg(feature = "gst")]
             BackendSelect::GStreamer => Self::new_gstreamer(config, cmd_tx),
-            BackendSelect::Default => Self::new_default(config, cmd_tx),
+            BackendSelect::Rusty => Self::new_rusty(config, cmd_tx),
         }
     }
 
-    /// Create a new Backend with default backend ordering
-    ///
-    /// For the order see [`BackendSelect::Default`]
-    #[allow(unreachable_code)]
-    fn new_default(config: &ServerOverlay, cmd_tx: PlayerCmdSender) -> Self {
-        #[cfg(feature = "rusty")]
-        return Self::new_rusty(config, cmd_tx);
-        #[cfg(feature = "gst")]
-        return Self::new_gstreamer(config, cmd_tx);
-        #[cfg(feature = "mpv")]
-        return Self::new_mpv(config, cmd_tx);
-
-        #[cfg(not(any(feature = "rusty", feature = "gst", feature = "mpv")))]
-        unimplemented!("No compiled backend!");
-    }
+    // /// Create a new Backend with default backend ordering
+    // ///
+    // /// For the order see [`BackendSelect::Default`]
+    // #[allow(unreachable_code)]
+    // fn new_default(config: &ServerOverlay, cmd_tx: PlayerCmdSender) -> Self {
+    //     #[cfg(feature = "gst")]
+    //     return Self::new_gstreamer(config, cmd_tx);
+    //     #[cfg(feature = "mpv")]
+    //     return Self::new_mpv(config, cmd_tx);
+    //     return Self::new_rusty(config, cmd_tx);
+    // }
 
     /// Explicitly choose Backend [`RustyBackend`](rusty_backend::RustyBackend)
-    #[cfg(feature = "rusty")]
     fn new_rusty(config: &ServerOverlay, cmd_tx: PlayerCmdSender) -> Self {
         info!("Using Backend \"rusty\"");
         Self::Rusty(rusty_backend::RustyBackend::new(config, cmd_tx))
@@ -165,12 +178,9 @@ impl Backend {
         match self {
             #[cfg(feature = "mpv")]
             Backend::Mpv(v) => v,
-            #[cfg(feature = "rusty")]
-            Backend::Rusty(v) => v,
             #[cfg(feature = "gst")]
             Backend::GStreamer(v) => v,
-            #[cfg(not(any(feature = "rusty", feature = "gst", feature = "mpv")))]
-            _ => unimplemented!("No compiled backend!"),
+            Backend::Rusty(v) => v,
         }
     }
 
@@ -179,17 +189,14 @@ impl Backend {
         match self {
             #[cfg(feature = "mpv")]
             Backend::Mpv(v) => v,
-            #[cfg(feature = "rusty")]
-            Backend::Rusty(v) => v,
             #[cfg(feature = "gst")]
             Backend::GStreamer(v) => v,
-            #[cfg(not(any(feature = "rusty", feature = "gst", feature = "mpv")))]
-            _ => unimplemented!("No compiled backend!"),
+            Backend::Rusty(v) => v,
         }
     }
 }
 
-#[derive(Clone, Serialize, Deserialize, Debug)]
+#[derive(Clone, Debug)]
 pub enum PlayerCmd {
     AboutToFinish,
     CycleLoop,
@@ -199,7 +206,6 @@ pub enum PlayerCmd {
     SkipPrevious,
     Pause,
     Play,
-    ProcessID,
     Quit,
     ReloadConfig,
     ReloadPlaylist,
@@ -215,6 +221,8 @@ pub enum PlayerCmd {
     VolumeUp,
 }
 
+pub type StreamTX = broadcast::Sender<UpdateEvents>;
+
 #[allow(clippy::module_name_repetitions)]
 pub struct GeneralPlayer {
     pub backend: Backend,
@@ -226,6 +234,7 @@ pub struct GeneralPlayer {
     pub db: DataBase,
     pub db_podcast: DBPod,
     pub cmd_tx: PlayerCmdSender,
+    pub stream_tx: StreamTX,
 }
 
 impl GeneralPlayer {
@@ -237,18 +246,19 @@ impl GeneralPlayer {
     /// - if config path creation fails
     pub fn new_backend(
         backend: BackendSelect,
-        config: ServerOverlay,
+        config: SharedServerSettings,
         cmd_tx: PlayerCmdSender,
+        stream_tx: StreamTX,
     ) -> Result<Self> {
-        let backend = Backend::new_select(backend, &config, cmd_tx.clone());
+        let config_read = config.read();
+        let backend = Backend::new_select(backend, &config_read, cmd_tx.clone());
 
         let db_path = get_app_config_path().with_context(|| "failed to get podcast db path.")?;
 
         let db_podcast = DBPod::new(&db_path).with_context(|| "error connecting to podcast db.")?;
-        let db = DataBase::new(&config)?;
+        let db = DataBase::new(&config_read)?;
 
-        let config = new_shared_server_settings(config);
-        let playlist = Playlist::new(config.clone()).unwrap_or_default();
+        let playlist = Playlist::new(&config).context("Failed to load playlist")?;
         let mpris = if config.read().settings.player.use_mediacontrols {
             Some(mpris::Mpris::new(cmd_tx.clone()))
         } else {
@@ -260,6 +270,8 @@ impl GeneralPlayer {
             None
         };
 
+        drop(config_read);
+
         Ok(Self {
             backend,
             playlist,
@@ -269,6 +281,7 @@ impl GeneralPlayer {
             db,
             db_podcast,
             cmd_tx,
+            stream_tx,
             current_track_updated: false,
         })
     }
@@ -279,8 +292,12 @@ impl GeneralPlayer {
     ///
     /// - if connecting to the database fails
     /// - if config path creation fails
-    pub fn new(config: ServerOverlay, cmd_tx: PlayerCmdSender) -> Result<Self> {
-        Self::new_backend(BackendSelect::Default, config, cmd_tx)
+    pub fn new(
+        config: SharedServerSettings,
+        cmd_tx: PlayerCmdSender,
+        stream_tx: StreamTX,
+    ) -> Result<Self> {
+        Self::new_backend(BackendSelect::Rusty, config, cmd_tx, stream_tx)
     }
 
     /// Reload the config from file, on fail continue to use the old
@@ -311,7 +328,7 @@ impl GeneralPlayer {
 
         if config.get_discord_status_enable() && self.discord.is_none() {
             // start discord ipc if new config has it enabled, but is not active yet
-            let mut discord = discord::Rpc::default();
+            let discord = discord::Rpc::default();
 
             // actually set the metadata of the currently playing track, otherwise the controls will work but no title or coverart will be set until next track
             if let Some(track) = self.playlist.current_track() {
@@ -338,13 +355,17 @@ impl GeneralPlayer {
     }
 
     pub fn toggle_gapless(&mut self) -> bool {
-        let new_gapless = !self.backend.as_player().gapless();
-        self.backend.as_player_mut().set_gapless(new_gapless);
+        let new_gapless = !<Self as PlayerTrait>::gapless(self);
+        <Self as PlayerTrait>::set_gapless(self, new_gapless);
         self.config.write().settings.player.gapless = new_gapless;
         new_gapless
     }
 
     /// Requires that the function is called on a thread with a entered tokio runtime
+    ///
+    /// # Panics
+    ///
+    /// if `current_track_index` in playlist is above u32
     pub fn start_play(&mut self) {
         if self.playlist.is_stopped() | self.playlist.is_paused() {
             self.playlist.set_status(Status::Running);
@@ -352,16 +373,13 @@ impl GeneralPlayer {
 
         self.playlist.proceed();
 
-        if let Some(track) = self.playlist.current_track() {
-            let track = track.clone();
-
+        if let Some(track) = self.playlist.current_track().cloned() {
             info!("Starting Track {:#?}", track);
 
             if self.playlist.has_next_track() {
                 self.playlist.set_next_track(None);
                 self.current_track_updated = true;
                 info!("gapless next track played");
-                #[cfg(feature = "rusty")]
                 #[allow(irrefutable_let_patterns)]
                 if let Backend::Rusty(ref mut backend) = self.backend {
                     backend.message_on_end();
@@ -378,11 +396,18 @@ impl GeneralPlayer {
 
             self.add_and_play_mpris_discord();
             self.player_restore_last_position();
-            #[cfg(feature = "rusty")]
             #[allow(irrefutable_let_patterns)]
             if let Backend::Rusty(ref mut backend) = self.backend {
                 backend.message_on_end();
             }
+
+            self.send_stream_ev(UpdateEvents::TrackChanged(TrackChangedInfo {
+                current_track_index: u64::try_from(self.playlist.get_current_track_index())
+                    .unwrap(),
+                current_track_updated: self.current_track_updated,
+                title: self.media_info().media_title,
+                progress: self.get_progress(),
+            }));
         }
     }
 
@@ -392,101 +417,73 @@ impl GeneralPlayer {
                 mpris.add_and_play(track);
             }
 
-            if let Some(ref mut discord) = self.discord {
+            if let Some(ref discord) = self.discord {
                 discord.update(track);
             }
         }
     }
     pub fn enqueue_next_from_playlist(&mut self) {
-        if self.playlist.next_track().is_some() {
+        if self.playlist.has_next_track() {
             return;
         }
 
-        let track = match self.playlist.fetch_next_track() {
-            Some(t) => t.clone(),
-            None => return,
+        let Some(track) = self.playlist.fetch_next_track().cloned() else {
+            return;
         };
 
-        self.playlist.set_next_track(Some(&track));
-        self.get_player_mut().enqueue_next(&track);
+        self.enqueue_next(&track);
 
         info!("Next track enqueued: {:#?}", track);
     }
 
+    /// Skip to the next track, if there is one
     pub fn next(&mut self) {
         if self.playlist.current_track().is_some() {
             info!("skip route 1 which is in most cases.");
             self.playlist.set_next_track(None);
-            self.get_player_mut().skip_one();
+            self.skip_one();
         } else {
             info!("skip route 2 cause no current track.");
             self.stop();
-            // if let Err(e) = crate::audio_cmd::<()>(PlayerCmd::StartPlay, false) {
-            //     debug!("Error in skip route 2: {e}");
-            // }
         }
     }
+
+    /// Switch & Play the previous track in the playlist
     pub fn previous(&mut self) {
         self.playlist.previous();
         self.playlist.proceed_false();
         self.next();
     }
+
+    /// Resume playback if paused, pause playback if running
     pub fn toggle_pause(&mut self) {
         match self.playlist.status() {
             Status::Running => {
-                self.get_player_mut().pause();
-                if let Some(ref mut mpris) = self.mpris {
-                    mpris.pause();
-                }
-                if let Some(ref mut discord) = self.discord {
-                    discord.pause();
-                }
-                self.playlist.set_status(Status::Paused);
+                <Self as PlayerTrait>::pause(self);
             }
             Status::Stopped => {}
             Status::Paused => {
-                self.get_player_mut().resume();
-                if let Some(ref mut mpris) = self.mpris {
-                    mpris.resume();
-                }
-                let time_pos = self.get_player().position();
-                if let Some(ref mut discord) = self.discord {
-                    discord.resume(time_pos);
-                }
-                self.playlist.set_status(Status::Running);
+                <Self as PlayerTrait>::resume(self);
             }
         }
     }
 
+    /// Pause playback if running
     pub fn pause(&mut self) {
         match self.playlist.status() {
             Status::Running => {
-                self.get_player_mut().pause();
-                if let Some(ref mut mpris) = self.mpris {
-                    mpris.pause();
-                }
-                if let Some(ref mut discord) = self.discord {
-                    discord.pause();
-                }
-                self.playlist.set_status(Status::Paused);
+                <Self as PlayerTrait>::pause(self);
             }
             Status::Stopped | Status::Paused => {}
         }
     }
 
+    /// Resume playback if paused
     pub fn play(&mut self) {
         match self.playlist.status() {
             Status::Running | Status::Stopped => {}
             Status::Paused => {
-                self.get_player_mut().resume();
-                if let Some(ref mut mpris) = self.mpris {
-                    mpris.resume();
-                }
-                let time_pos = self.get_player().position();
-                if let Some(ref mut discord) = self.discord {
-                    discord.resume(time_pos);
-                }
-                self.playlist.set_status(Status::Running);
+                <Self as PlayerTrait>::resume(self);
             }
         }
     }
@@ -512,9 +509,7 @@ impl GeneralPlayer {
         if !forward {
             offset = -offset;
         }
-        self.get_player_mut()
-            .seek(offset)
-            .expect("Error in player seek.");
+        self.seek(offset).expect("Error in player seek.");
     }
 
     #[allow(clippy::cast_sign_loss)]
@@ -523,7 +518,7 @@ impl GeneralPlayer {
             info!("Not saving Last position as there is no current track");
             return;
         };
-        let Some(position) = self.get_player().position() else {
+        let Some(position) = self.position() else {
             info!("Not saving Last position as there is no position");
             return;
         };
@@ -581,13 +576,13 @@ impl GeneralPlayer {
             match track.media_type {
                 MediaType::Music => {
                     if let Ok(last_pos) = self.db.get_last_position(track) {
-                        self.get_player_mut().seek_to(last_pos);
+                        self.seek_to(last_pos);
                         restored = true;
                     }
                 }
                 MediaType::Podcast => {
                     if let Ok(last_pos) = self.db_podcast.get_last_position(track) {
-                        self.get_player_mut().seek_to(last_pos);
+                        self.seek_to(last_pos);
                         restored = true;
                     }
                 }
@@ -608,6 +603,14 @@ impl GeneralPlayer {
             }
         }
     }
+
+    /// Send stream events with consistent error handling
+    fn send_stream_ev(&self, ev: UpdateEvents) {
+        // there is only one error case: no receivers
+        if self.stream_tx.send(ev).is_err() {
+            debug!("Stream Event not send: No Receivers");
+        }
+    }
 }
 
 #[async_trait]
@@ -619,18 +622,47 @@ impl PlayerTrait for GeneralPlayer {
         self.get_player().volume()
     }
     fn add_volume(&mut self, volume: VolumeSigned) -> Volume {
-        self.get_player_mut().add_volume(volume)
+        let vol = self.get_player_mut().add_volume(volume);
+        self.send_stream_ev(UpdateEvents::VolumeChanged { volume: vol });
+
+        vol
     }
     fn set_volume(&mut self, volume: Volume) -> Volume {
-        self.get_player_mut().set_volume(volume)
+        let vol = self.get_player_mut().set_volume(volume);
+        self.send_stream_ev(UpdateEvents::VolumeChanged { volume: vol });
+
+        vol
     }
+    /// This function should not be used directly, use GeneralPlayer::pause
     fn pause(&mut self) {
         self.playlist.set_status(Status::Paused);
         self.get_player_mut().pause();
+        if let Some(ref mut mpris) = self.mpris {
+            mpris.pause();
+        }
+        if let Some(ref discord) = self.discord {
+            discord.pause();
+        }
+
+        self.send_stream_ev(UpdateEvents::PlayStateChanged {
+            playing: Status::Paused.as_u32(),
+        });
     }
+    /// This function should not be used directly, use GeneralPlayer::play
     fn resume(&mut self) {
         self.playlist.set_status(Status::Running);
         self.get_player_mut().resume();
+        if let Some(ref mut mpris) = self.mpris {
+            mpris.resume();
+        }
+        let time_pos = self.get_player().position();
+        if let Some(ref discord) = self.discord {
+            discord.resume(time_pos);
+        }
+
+        self.send_stream_ev(UpdateEvents::PlayStateChanged {
+            playing: Status::Running.as_u32(),
+        });
     }
     fn is_paused(&self) -> bool {
         self.get_player().is_paused()
@@ -643,11 +675,17 @@ impl PlayerTrait for GeneralPlayer {
     }
 
     fn set_speed(&mut self, speed: Speed) -> Speed {
-        self.get_player_mut().set_speed(speed)
+        let speed = self.get_player_mut().set_speed(speed);
+        self.send_stream_ev(UpdateEvents::SpeedChanged { speed });
+
+        speed
     }
 
     fn add_speed(&mut self, speed: SpeedSigned) -> Speed {
-        self.get_player_mut().add_speed(speed)
+        let speed = self.get_player_mut().add_speed(speed);
+        self.send_stream_ev(UpdateEvents::SpeedChanged { speed });
+
+        speed
     }
 
     fn speed(&self) -> Speed {
@@ -655,9 +693,7 @@ impl PlayerTrait for GeneralPlayer {
     }
 
     fn stop(&mut self) {
-        self.playlist.set_status(Status::Stopped);
-        self.playlist.set_next_track(None);
-        self.playlist.clear_current_track();
+        self.playlist.stop();
         self.get_player_mut().stop();
     }
 
@@ -671,6 +707,7 @@ impl PlayerTrait for GeneralPlayer {
 
     fn set_gapless(&mut self, to: bool) {
         self.get_player_mut().set_gapless(to);
+        self.send_stream_ev(UpdateEvents::GaplessChanged { gapless: to });
     }
 
     fn skip_one(&mut self) {
@@ -690,35 +727,6 @@ impl PlayerTrait for GeneralPlayer {
     }
 }
 
-/// The primitive in which time (current position / total duration) will be stored as
-pub type PlayerTimeUnit = Duration;
-
-/// Struct to keep both values with a name, as tuples cannot have named fields
-#[derive(Debug, Clone, Copy, PartialEq)]
-pub struct PlayerProgress {
-    pub position: Option<PlayerTimeUnit>,
-    /// Total duration of the currently playing track, if there is a known total duration
-    pub total_duration: Option<PlayerTimeUnit>,
-}
-
-impl From<crate::player::PlayerTime> for PlayerProgress {
-    fn from(value: crate::player::PlayerTime) -> Self {
-        Self {
-            position: value.position.map(std::convert::Into::into),
-            total_duration: value.total_duration.map(std::convert::Into::into),
-        }
-    }
-}
-
-impl From<PlayerProgress> for crate::player::PlayerTime {
-    fn from(value: PlayerProgress) -> Self {
-        Self {
-            position: value.position.map(std::convert::Into::into),
-            total_duration: value.total_duration.map(std::convert::Into::into),
-        }
-    }
-}
-
 /// Some information that may be available from the backend
 /// This is different from [`Track`] as this is everything parsed from the decoder's metadata
 /// and [`Track`] stores some different extra stuff
