@@ -184,7 +184,7 @@ impl Symphonia {
             &mut *decoder,
             BufferInputType::New(&mut buffer),
             track_id,
-            &time_base,
+            time_base,
             &mut media_title_tx,
             &mut probed.metadata,
             &mut None,
@@ -270,16 +270,11 @@ impl Source for Symphonia {
     }
 
     #[inline]
-    fn elapsed(&mut self) -> Duration {
-        self.elapsed
-    }
-
-    #[inline]
-    fn seek(&mut self, time: Duration) -> Option<Duration> {
+    fn try_seek(&mut self, pos: Duration) -> Result<(), rodio::source::SeekError> {
         match self.probed.format.seek(
             SeekMode::Coarse,
             SeekTo::Time {
-                time: time.into(),
+                time: pos.into(),
                 track_id: Some(self.track_id),
             },
         ) {
@@ -300,11 +295,9 @@ impl Source for Symphonia {
                     self.decoder.reset();
                 }
 
-                self.time_base
-                    .as_ref()
-                    .map(|v| v.calc_time(seeked_to.actual_ts).into())
+                Ok(())
             }
-            Err(_) => None,
+            Err(_) => Ok(()),
         }
     }
 }
@@ -320,7 +313,7 @@ impl Iterator for Symphonia {
                 &mut *self.decoder,
                 BufferInputType::Existing(&mut self.buffer),
                 self.track_id,
-                &self.time_base,
+                self.time_base,
                 &mut self.media_title_tx,
                 &mut self.probed.metadata,
                 &mut self.seek_required_ts,
@@ -416,7 +409,7 @@ fn decode_loop(
     decoder: &mut dyn codecs::Decoder,
     buffer: BufferInputType<'_>,
     track_id: u32,
-    time_base: &Option<TimeBase>,
+    time_base: Option<TimeBase>,
     media_title_tx: &mut MediaTitleTxWrap,
     probed: &mut ProbedMetadata,
     seek_required_ts: &mut Option<NonZeroU64>,
@@ -455,7 +448,7 @@ fn decode_loop(
     if !media_title_tx.is_none() {
         // run container metadata if new and on seek to 0
         // this maybe not be 100% reliable, where for example there is no "time_base", but i dont know of a case yet where that happens
-        if elapsed.as_ref().map_or(false, Duration::is_zero) {
+        if elapsed.as_ref().is_some_and(Duration::is_zero) {
             trace!("Time is 0, doing container metadata");
             media_title_tx.send_reset();
             do_container_metdata(media_title_tx, format, probed);
