@@ -1,25 +1,16 @@
 #![cfg_attr(test, deny(missing_docs))]
 
-mod conversions;
+mod decoder;
 mod icy_metadata;
-#[allow(unused)]
 mod sink;
-mod stream;
-
-pub mod buffer;
-pub mod decoder;
-pub mod dynamic_mixer;
-pub mod queue;
-pub mod source;
+mod source;
 
 use async_trait::async_trait;
-pub use conversions::Sample;
-pub use cpal::{traits::StreamTrait, ChannelCount, SampleRate};
-pub use decoder::Symphonia;
-pub use sink::Sink;
-pub use source::Source;
+use decoder::Symphonia;
+use rodio::OutputStream;
+use rodio::Source;
+use sink::Sink;
 use std::num::{NonZeroU16, NonZeroUsize};
-pub use stream::OutputStream;
 use termusiclib::config::ServerOverlay;
 use tokio::runtime::Handle;
 
@@ -58,7 +49,6 @@ use termusiclib::track::{MediaType, Track};
 pub type TotalDuration = Option<Duration>;
 pub type ArcTotalDuration = Arc<Mutex<TotalDuration>>;
 
-#[allow(unused)]
 #[derive(Clone, Debug)]
 pub enum PlayerInternalCmd {
     MessageOnEnd,
@@ -432,7 +422,7 @@ async fn player_thread(
     // note that the current implementation is only meant to have 1 enqueued next after the current playing song
     let mut next_duration_opt = None;
     let (_stream, handle) = OutputStream::try_default().unwrap();
-    let mut sink = Sink::try_new(&handle, picmd_tx.clone(), pcmd_tx.clone()).unwrap();
+    let sink = Sink::try_new(&handle, picmd_tx.clone(), pcmd_tx.clone()).unwrap();
     sink.set_speed(speed_inside as f32 / 10.0);
     sink.set_volume(f32::from(volume_inside.load(Ordering::SeqCst)) / 100.0);
     loop {
@@ -487,9 +477,7 @@ async fn player_thread(
                 sink.set_speed(speed_inside as f32 / 10.0);
             }
             PlayerInternalCmd::Stop => {
-                sink = Sink::try_new(&handle, picmd_tx.clone(), pcmd_tx.clone()).unwrap();
-                sink.set_speed(speed_inside as f32 / 10.0);
-                sink.set_volume(f32::from(volume_inside.load(Ordering::SeqCst)) / 100.0);
+                sink.stop();
             }
             PlayerInternalCmd::Volume(volume) => {
                 sink.set_volume(f32::from(volume) / 100.0);
@@ -497,7 +485,7 @@ async fn player_thread(
             }
             PlayerInternalCmd::Skip => {
                 // the sink can be empty, if for example nothing could be enqueued, so a "skip_one" would be a no-op and never send EOS, which is required to go to the next track
-                if sink.empty() {
+                if sink.is_empty() {
                     let _ = picmd_tx.send(PlayerInternalCmd::Eos);
                     let _ = pcmd_tx.send(PlayerCmd::Eos);
                 } else {
@@ -756,14 +744,3 @@ async fn queue_next(
         }
     }
 }
-
-// Parse a given `str` int oa [`reqwest::Url`], otherwise log a error
-// fn parse_url(url_str: &str) -> Option<reqwest::Url> {
-//     match url_str.parse::<reqwest::Url>() {
-//         Ok(v) => Some(v),
-//         Err(err) => {
-//             error!("error parse url: {:#?}", err);
-//             None
-//         }
-//     }
-// }
