@@ -1,3 +1,4 @@
+use anyhow::Result;
 /**
  * MIT License
  *
@@ -25,16 +26,18 @@ use std::convert::From;
 use termusiclib::config::v2::tui::theme::styles::ColorTermusic;
 use termusiclib::config::v2::tui::theme::ThemeWrap;
 use termusiclib::config::SharedTuiSettings;
-use termusiclib::types::{ConfigEditorMsg, IdConfigEditor, Msg};
+use termusiclib::types::{ConfigEditorMsg, Id, IdConfigEditor, Msg};
 use tui_realm_stdlib::{Input, Label, Select, Table};
 use tuirealm::command::{Cmd, CmdResult, Direction, Position};
 use tuirealm::event::{Key, KeyEvent, KeyModifiers, NoUserEvent};
 use tuirealm::props::{
     Alignment, BorderType, Borders, Color, InputType, Style, TableBuilder, TextModifiers, TextSpan,
 };
-use tuirealm::tui::style::Modifier;
+use tuirealm::ratatui::style::Modifier;
 use tuirealm::{AttrValue, Attribute, Component, Event, MockComponent, State, StateValue};
 
+use crate::ui::model::Model;
+
 const COLOR_LIST: [ColorTermusic; 19] = [
     ColorTermusic::Reset,
     ColorTermusic::Foreground,
@@ -102,7 +105,7 @@ impl Component<Msg, NoUserEvent> for CEThemeSelectTable {
     fn on(&mut self, ev: Event<NoUserEvent>) -> Option<Msg> {
         let config = self.config.clone();
         let keys = &config.read().settings.keys;
-        let _cmd_result = match ev {
+        let cmd_result = match ev {
             // Global Hotkeys
             Event::Keyboard(keyevent) if keyevent == keys.config_keys.save.get() => {
                 return Some(Msg::ConfigEditor(ConfigEditorMsg::CloseOk));
@@ -168,7 +171,10 @@ impl Component<Msg, NoUserEvent> for CEThemeSelectTable {
 
             _ => CmdResult::None,
         };
-        Some(Msg::None)
+        match cmd_result {
+            CmdResult::None => None,
+            _ => Some(Msg::ForceRedraw),
+        }
     }
 }
 
@@ -191,7 +197,7 @@ impl CEColorSelect {
         on_key_backshift: Msg,
     ) -> Self {
         let init_value = Self::init_color_select(id, &config.read().settings.theme);
-        let mut choices = vec![];
+        let mut choices = Vec::new();
         for color in &COLOR_LIST {
             choices.push(color.as_ref());
         }
@@ -291,7 +297,7 @@ impl CEColorSelect {
                 AttrValue::Style(Style::default().add_modifier(Modifier::BOLD).bg(Color::Red)),
             );
 
-            Msg::None
+            Msg::ForceRedraw
         }
     }
 }
@@ -349,7 +355,8 @@ impl Component<Msg, NoUserEvent> for CEColorSelect {
             CmdResult::Submit(State::One(StateValue::Usize(index))) => {
                 Some(self.update_color(index))
             }
-            _ => Some(Msg::None),
+            CmdResult::None => None,
+            _ => Some(Msg::ForceRedraw),
         }
     }
 }
@@ -866,7 +873,7 @@ impl ConfigInputHighlight {
             if symbol.is_empty() {
                 let color = self.config.read().settings.theme.library_border();
                 self.update_symbol_after(color);
-                return Msg::None;
+                return Msg::ForceRedraw;
             }
             if let Some(s) = Self::string_to_unicode_char(&symbol) {
                 // success getting a unicode letter
@@ -885,7 +892,7 @@ impl ConfigInputHighlight {
                 return Msg::ConfigEditor(ConfigEditorMsg::SymbolChanged(self.id, s.to_string()));
             }
         }
-        Msg::None
+        Msg::ForceRedraw
     }
     fn update_symbol_after(&mut self, color: Color) {
         self.attr(Attribute::Foreground, AttrValue::Color(color));
@@ -923,30 +930,27 @@ impl Component<Msg, NoUserEvent> for ConfigInputHighlight {
             Event::Keyboard(keyevent) if keyevent == keys.escape.get() => {
                 Some(Msg::ConfigEditor(ConfigEditorMsg::CloseCancel))
             }
-            // Event::Keyboard(keyevent) if keyevent == keys.global_quit.get() => {
-            //     Some(Msg::ConfigEditor(ConfigEditorMsg::CloseCancel))
-            // }
             Event::Keyboard(KeyEvent {
                 code: Key::Left, ..
             }) => {
                 self.perform(Cmd::Move(Direction::Left));
-                Some(Msg::None)
+                Some(Msg::ForceRedraw)
             }
             Event::Keyboard(KeyEvent {
                 code: Key::Right, ..
             }) => {
                 self.perform(Cmd::Move(Direction::Right));
-                Some(Msg::None)
+                Some(Msg::ForceRedraw)
             }
             Event::Keyboard(KeyEvent {
                 code: Key::Home, ..
             }) => {
                 self.perform(Cmd::GoTo(Position::Begin));
-                Some(Msg::None)
+                Some(Msg::ForceRedraw)
             }
             Event::Keyboard(KeyEvent { code: Key::End, .. }) => {
                 self.perform(Cmd::GoTo(Position::End));
-                Some(Msg::None)
+                Some(Msg::ForceRedraw)
             }
             Event::Keyboard(KeyEvent {
                 code: Key::Delete, ..
@@ -981,7 +985,7 @@ impl Component<Msg, NoUserEvent> for ConfigInputHighlight {
                 IdConfigEditor::CurrentlyPlayingTrackSymbol => Some(Msg::ConfigEditor(
                     ConfigEditorMsg::CurrentlyPlayingTrackSymbolBlurDown,
                 )),
-                _ => Some(Msg::None),
+                _ => None,
             },
             Event::Keyboard(KeyEvent { code: Key::Up, .. }) => match self.id {
                 IdConfigEditor::LibraryHighlightSymbol => Some(Msg::ConfigEditor(
@@ -993,7 +997,7 @@ impl Component<Msg, NoUserEvent> for ConfigInputHighlight {
                 IdConfigEditor::CurrentlyPlayingTrackSymbol => Some(Msg::ConfigEditor(
                     ConfigEditorMsg::CurrentlyPlayingTrackSymbolBlurUp,
                 )),
-                _ => Some(Msg::None),
+                _ => None,
             },
 
             Event::Keyboard(KeyEvent {
@@ -1002,7 +1006,7 @@ impl Component<Msg, NoUserEvent> for ConfigInputHighlight {
                 let result = self.perform(Cmd::Submit);
                 Some(self.update_symbol(result))
             }
-            _ => Some(Msg::None),
+            _ => None,
         }
     }
 }
@@ -1306,3 +1310,284 @@ impl Component<Msg, NoUserEvent> for ConfigFallbackHighlight {
         self.component.on(ev)
     }
 }
+
+impl Model {
+    /// Mount / Remount the Config-Editor's Second Page, the Theme, Color & Symbol Options
+    #[allow(clippy::too_many_lines)]
+    pub(super) fn remount_config_color(
+        &mut self,
+        config: &SharedTuiSettings,
+        theme_idx: Option<usize>,
+    ) -> Result<()> {
+        // Mount color page
+        self.app.remount(
+            Id::ConfigEditor(IdConfigEditor::CEThemeSelect),
+            Box::new(CEThemeSelectTable::new(config.clone())),
+            Vec::new(),
+        )?;
+
+        self.app.remount(
+            Id::ConfigEditor(IdConfigEditor::LibraryLabel),
+            Box::<ConfigLibraryTitle>::default(),
+            Vec::new(),
+        )?;
+        self.app.remount(
+            Id::ConfigEditor(IdConfigEditor::LibraryForeground),
+            Box::new(ConfigLibraryForeground::new(config.clone())),
+            Vec::new(),
+        )?;
+        self.app.remount(
+            Id::ConfigEditor(IdConfigEditor::LibraryBackground),
+            Box::new(ConfigLibraryBackground::new(config.clone())),
+            Vec::new(),
+        )?;
+        self.app.remount(
+            Id::ConfigEditor(IdConfigEditor::LibraryBorder),
+            Box::new(ConfigLibraryBorder::new(config.clone())),
+            Vec::new(),
+        )?;
+        self.app.remount(
+            Id::ConfigEditor(IdConfigEditor::LibraryHighlight),
+            Box::new(ConfigLibraryHighlight::new(config.clone())),
+            Vec::new(),
+        )?;
+
+        self.app.remount(
+            Id::ConfigEditor(IdConfigEditor::PlaylistLabel),
+            Box::<ConfigPlaylistTitle>::default(),
+            Vec::new(),
+        )?;
+
+        self.app.remount(
+            Id::ConfigEditor(IdConfigEditor::PlaylistForeground),
+            Box::new(ConfigPlaylistForeground::new(config.clone())),
+            Vec::new(),
+        )?;
+
+        self.app.remount(
+            Id::ConfigEditor(IdConfigEditor::PlaylistBackground),
+            Box::new(ConfigPlaylistBackground::new(config.clone())),
+            Vec::new(),
+        )?;
+
+        self.app.remount(
+            Id::ConfigEditor(IdConfigEditor::PlaylistBorder),
+            Box::new(ConfigPlaylistBorder::new(config.clone())),
+            Vec::new(),
+        )?;
+
+        self.app.remount(
+            Id::ConfigEditor(IdConfigEditor::PlaylistHighlight),
+            Box::new(ConfigPlaylistHighlight::new(config.clone())),
+            Vec::new(),
+        )?;
+
+        self.app.remount(
+            Id::ConfigEditor(IdConfigEditor::ProgressLabel),
+            Box::<ConfigProgressTitle>::default(),
+            Vec::new(),
+        )?;
+
+        self.app.remount(
+            Id::ConfigEditor(IdConfigEditor::ProgressForeground),
+            Box::new(ConfigProgressForeground::new(config.clone())),
+            Vec::new(),
+        )?;
+
+        self.app.remount(
+            Id::ConfigEditor(IdConfigEditor::ProgressBackground),
+            Box::new(ConfigProgressBackground::new(config.clone())),
+            Vec::new(),
+        )?;
+
+        self.app.remount(
+            Id::ConfigEditor(IdConfigEditor::ProgressBorder),
+            Box::new(ConfigProgressBorder::new(config.clone())),
+            Vec::new(),
+        )?;
+
+        self.app.remount(
+            Id::ConfigEditor(IdConfigEditor::LyricLabel),
+            Box::<ConfigLyricTitle>::default(),
+            Vec::new(),
+        )?;
+        self.app.remount(
+            Id::ConfigEditor(IdConfigEditor::LyricForeground),
+            Box::new(ConfigLyricForeground::new(config.clone())),
+            Vec::new(),
+        )?;
+        self.app.remount(
+            Id::ConfigEditor(IdConfigEditor::LyricBackground),
+            Box::new(ConfigLyricBackground::new(config.clone())),
+            Vec::new(),
+        )?;
+        self.app.remount(
+            Id::ConfigEditor(IdConfigEditor::LyricBorder),
+            Box::new(ConfigLyricBorder::new(config.clone())),
+            Vec::new(),
+        )?;
+
+        self.app.remount(
+            Id::ConfigEditor(IdConfigEditor::ImportantPopupLabel),
+            Box::<ConfigImportantPopupTitle>::default(),
+            Vec::new(),
+        )?;
+        self.app.remount(
+            Id::ConfigEditor(IdConfigEditor::ImportantPopupForeground),
+            Box::new(ConfigImportantPopupForeground::new(config.clone())),
+            Vec::new(),
+        )?;
+        self.app.remount(
+            Id::ConfigEditor(IdConfigEditor::ImportantPopupBackground),
+            Box::new(ConfigImportantPopupBackground::new(config.clone())),
+            Vec::new(),
+        )?;
+        self.app.remount(
+            Id::ConfigEditor(IdConfigEditor::ImportantPopupBorder),
+            Box::new(ConfigImportantPopupBorder::new(config.clone())),
+            Vec::new(),
+        )?;
+
+        self.app.remount(
+            Id::ConfigEditor(IdConfigEditor::FallbackLabel),
+            Box::<ConfigFallbackTitle>::default(),
+            Vec::new(),
+        )?;
+        self.app.remount(
+            Id::ConfigEditor(IdConfigEditor::FallbackForeground),
+            Box::new(ConfigFallbackForeground::new(config.clone())),
+            Vec::new(),
+        )?;
+        self.app.remount(
+            Id::ConfigEditor(IdConfigEditor::FallbackBackground),
+            Box::new(ConfigFallbackBackground::new(config.clone())),
+            Vec::new(),
+        )?;
+        self.app.remount(
+            Id::ConfigEditor(IdConfigEditor::FallbackBorder),
+            Box::new(ConfigFallbackBorder::new(config.clone())),
+            Vec::new(),
+        )?;
+        self.app.remount(
+            Id::ConfigEditor(IdConfigEditor::FallbackHighlight),
+            Box::new(ConfigFallbackHighlight::new(config.clone())),
+            Vec::new(),
+        )?;
+
+        self.remount_config_color_symbols(config)?;
+
+        self.theme_select_sync(theme_idx);
+
+        Ok(())
+    }
+
+    /// Mount / Remount the Config-Editor's Second Page, Symbols
+    fn remount_config_color_symbols(&mut self, config: &SharedTuiSettings) -> Result<()> {
+        self.app.remount(
+            Id::ConfigEditor(IdConfigEditor::LibraryHighlightSymbol),
+            Box::new(ConfigLibraryHighlightSymbol::new(config.clone())),
+            Vec::new(),
+        )?;
+
+        self.app.remount(
+            Id::ConfigEditor(IdConfigEditor::PlaylistHighlightSymbol),
+            Box::new(ConfigPlaylistHighlightSymbol::new(config.clone())),
+            Vec::new(),
+        )?;
+
+        self.app.remount(
+            Id::ConfigEditor(IdConfigEditor::CurrentlyPlayingTrackSymbol),
+            Box::new(ConfigCurrentlyPlayingTrackSymbol::new(config.clone())),
+            Vec::new(),
+        )?;
+
+        Ok(())
+    }
+
+    /// Unmount the Config-Editor's Second Page, the Theme, Color & Symbol Options
+    pub(super) fn umount_config_color(&mut self) -> Result<()> {
+        self.app
+            .umount(&Id::ConfigEditor(IdConfigEditor::CEThemeSelect))?;
+
+        self.app
+            .umount(&Id::ConfigEditor(IdConfigEditor::LibraryLabel))?;
+        self.app
+            .umount(&Id::ConfigEditor(IdConfigEditor::LibraryForeground))?;
+
+        self.app
+            .umount(&Id::ConfigEditor(IdConfigEditor::LibraryBackground))?;
+        self.app
+            .umount(&Id::ConfigEditor(IdConfigEditor::LibraryBorder))?;
+        self.app
+            .umount(&Id::ConfigEditor(IdConfigEditor::LibraryHighlight))?;
+
+        self.app
+            .umount(&Id::ConfigEditor(IdConfigEditor::PlaylistLabel))?;
+
+        self.app
+            .umount(&Id::ConfigEditor(IdConfigEditor::PlaylistForeground))?;
+
+        self.app
+            .umount(&Id::ConfigEditor(IdConfigEditor::PlaylistBackground))?;
+        self.app
+            .umount(&Id::ConfigEditor(IdConfigEditor::PlaylistBorder))?;
+        self.app
+            .umount(&Id::ConfigEditor(IdConfigEditor::PlaylistHighlight))?;
+        self.app
+            .umount(&Id::ConfigEditor(IdConfigEditor::ProgressLabel))?;
+
+        self.app
+            .umount(&Id::ConfigEditor(IdConfigEditor::ProgressForeground))?;
+        self.app
+            .umount(&Id::ConfigEditor(IdConfigEditor::ProgressBackground))?;
+        self.app
+            .umount(&Id::ConfigEditor(IdConfigEditor::ProgressBorder))?;
+
+        self.app
+            .umount(&Id::ConfigEditor(IdConfigEditor::LyricLabel))?;
+        self.app
+            .umount(&Id::ConfigEditor(IdConfigEditor::LyricForeground))?;
+        self.app
+            .umount(&Id::ConfigEditor(IdConfigEditor::LyricBackground))?;
+        self.app
+            .umount(&Id::ConfigEditor(IdConfigEditor::LyricBorder))?;
+
+        self.app
+            .umount(&Id::ConfigEditor(IdConfigEditor::ImportantPopupLabel))?;
+        self.app
+            .umount(&Id::ConfigEditor(IdConfigEditor::ImportantPopupForeground))?;
+        self.app
+            .umount(&Id::ConfigEditor(IdConfigEditor::ImportantPopupBackground))?;
+        self.app
+            .umount(&Id::ConfigEditor(IdConfigEditor::ImportantPopupBorder))?;
+
+        self.app
+            .umount(&Id::ConfigEditor(IdConfigEditor::FallbackLabel))?;
+        self.app
+            .umount(&Id::ConfigEditor(IdConfigEditor::FallbackForeground))?;
+
+        self.app
+            .umount(&Id::ConfigEditor(IdConfigEditor::FallbackBackground))?;
+        self.app
+            .umount(&Id::ConfigEditor(IdConfigEditor::FallbackBorder))?;
+        self.app
+            .umount(&Id::ConfigEditor(IdConfigEditor::FallbackHighlight))?;
+
+        self.umount_config_color_symbols()?;
+
+        Ok(())
+    }
+
+    /// Unmount the Config-Editor's Second Page, Symbols
+    pub fn umount_config_color_symbols(&mut self) -> Result<()> {
+        self.app
+            .umount(&Id::ConfigEditor(IdConfigEditor::LibraryHighlightSymbol))?;
+        self.app
+            .umount(&Id::ConfigEditor(IdConfigEditor::PlaylistHighlightSymbol))?;
+        self.app.umount(&Id::ConfigEditor(
+            IdConfigEditor::CurrentlyPlayingTrackSymbol,
+        ))?;
+
+        Ok(())
+    }
+}
