@@ -27,13 +27,12 @@ mod key_combo;
 mod update;
 mod view;
 
-use crate::ui::model::ConfigEditorLayout;
+use crate::ui::model::{ConfigEditorLayout, Model};
 use crate::ui::{ConfigEditorMsg, Msg};
-pub use color::*;
-pub use general::*;
-pub use key_combo::*;
+use anyhow::Result;
 
 use termusiclib::config::{SharedTuiSettings, TuiOverlay};
+use termusiclib::types::{Id, IdConfigEditor};
 use tui_realm_stdlib::{Radio, Span};
 use tuirealm::props::{Alignment, BorderSides, BorderType, Borders, Style, TextSpan};
 use tuirealm::{event::NoUserEvent, Component, Event, MockComponent};
@@ -147,3 +146,33 @@ impl Component<Msg, NoUserEvent> for ConfigSavePopup {
         )
     }
 }
+
+impl Model {
+    /// Mount / Remount the Config-Editor's Header & Footer
+    fn remount_config_header_footer(&mut self) -> Result<()> {
+        self.app.remount(
+            Id::ConfigEditor(IdConfigEditor::Header),
+            Box::new(CEHeader::new(
+                self.config_editor.layout,
+                &self.config_tui.read(),
+            )),
+            Vec::new(),
+        )?;
+        self.app.remount(
+            Id::ConfigEditor(IdConfigEditor::Footer),
+            Box::new(CEFooter::new(&self.config_tui.read())),
+            Vec::new(),
+        )?;
+
+        Ok(())
+    }
+
+    /// Unmount the Config-Editor's Header & Footer
+    fn umount_config_header_footer(&mut self) -> Result<()> {
+        self.app.umount(&Id::ConfigEditor(IdConfigEditor::Header))?;
+
+        self.app.umount(&Id::ConfigEditor(IdConfigEditor::Footer))?;
+
+        Ok(())
+    }
+}
