@@ -1,3 +1,4 @@
+use crate::ui::tui_cmd::TuiCmd;
 /**
  * MIT License
  *
@@ -23,7 +24,6 @@
  */
 use crate::ui::Model;
 use anyhow::Context;
-use std::path::PathBuf;
 use termusiclib::config::new_shared_tui_settings;
 use termusiclib::config::v2::server::config_extra::ServerConfigVersionedDefaulted;
 use termusiclib::config::v2::tui::config_extra::TuiConfigVersionedDefaulted;
@@ -31,7 +31,7 @@ use termusiclib::config::v2::tui::keys::KeyBinding;
 use termusiclib::config::v2::tui::theme::styles::ColorTermusic;
 use termusiclib::config::v2::tui::theme::ThemeColors;
 use termusiclib::types::{ConfigEditorMsg, Id, IdConfigEditor, IdKey, KFMsg, Msg};
-use termusicplayback::PlayerCmd;
+use termusiclib::utils::get_app_config_path;
 
 impl Model {
     #[allow(clippy::too_many_lines)]
@@ -170,7 +170,7 @@ impl Model {
                         }
 
                         if both_ok {
-                            self.command(&PlayerCmd::ReloadConfig);
+                            self.command(TuiCmd::ReloadConfig);
 
                             // only exit config editor if saving was successful
                             self.umount_config_editor();
@@ -339,22 +339,7 @@ impl Model {
             }
 
             ConfigEditorMsg::ThemeSelectLoad(index) => {
-                if let Some(theme_path_str) = self.config_editor.themes.get(index) {
-                    let theme_path = PathBuf::from(theme_path_str);
-                    if let Some(theme_file_stem) = theme_path.file_stem() {
-                        self.config_tui.write().settings.theme.theme.name =
-                            theme_file_stem.to_string_lossy().to_string();
-                        if let Ok(theme) = ThemeColors::from_yaml_file(&theme_path) {
-                            self.config_editor.theme.theme = theme;
-                        }
-                    }
-                }
-                self.config_editor.config_changed = true;
-                let mut config = self.config_tui.read().clone();
-                // This is for preview the theme colors
-                config.settings.theme = self.config_editor.theme.clone();
-                let config = new_shared_tui_settings(config);
-                self.remount_config_color(&config);
+                self.preview_theme(index);
             }
             ConfigEditorMsg::ColorChanged(id, color_config) => {
                 self.config_editor.config_changed = true;
@@ -365,16 +350,13 @@ impl Model {
 
                 match id {
                     IdConfigEditor::LibraryHighlightSymbol => {
-                        self.config_editor.theme.style.library.highlight_symbol =
-                            symbol.to_string();
+                        self.config_editor.theme.style.library.highlight_symbol = symbol;
                     }
                     IdConfigEditor::PlaylistHighlightSymbol => {
-                        self.config_editor.theme.style.playlist.highlight_symbol =
-                            symbol.to_string();
+                        self.config_editor.theme.style.playlist.highlight_symbol = symbol;
                     }
                     IdConfigEditor::CurrentlyPlayingTrackSymbol => {
-                        self.config_editor.theme.style.playlist.current_track_symbol =
-                            symbol.to_string();
+                        self.config_editor.theme.style.playlist.current_track_symbol = symbol;
                     }
                     _ => {}
                 };
@@ -386,6 +368,51 @@ impl Model {
         None
     }
 
+    /// Preview theme at Table index
+    fn preview_theme(&mut self, index: usize) {
+        // table entry 0 is termusic default
+        if index == 0 {
+            self.preview_theme_apply(ThemeColors::full_default(), 0);
+
+            return;
+        }
+
+        // idx - 1 as 0 table-entry is termusic default, which always exists
+        if let Some(theme_filename) = self.config_editor.themes.get(index - 1) {
+            match get_app_config_path() {
+                Ok(mut theme_path) => {
+                    theme_path.push("themes");
+                    theme_path.push(format!("{theme_filename}.yml"));
+                    match ThemeColors::from_yaml_file(&theme_path) {
+                        Ok(mut theme) => {
+                            theme.file_name = Some(theme_filename.to_string());
+
+                            self.preview_theme_apply(theme, index);
+                        }
+                        Err(e) => {
+                            error!("Failed to load theme colors: {:?}", e);
+                        }
+                    }
+                }
+                Err(e) => {
+                    error!("Error getting config path: {:?}", e);
+                }
+            }
+        }
+    }
+
+    /// Apply the given theme as a preview
+    fn preview_theme_apply(&mut self, theme: ThemeColors, index: usize) {
+        self.config_editor.theme.theme = theme;
+        self.config_editor.config_changed = true;
+
+        // This is for preview the theme colors
+        let mut config = self.config_tui.read().clone();
+        config.settings.theme = self.config_editor.theme.clone();
+        let config = new_shared_tui_settings(config);
+        self.remount_config_color(&config, Some(index)).unwrap();
+    }
+
     #[allow(clippy::too_many_lines)]
     fn update_key_focus(&mut self, msg: KFMsg) {
         match msg {
