@@ -1,13 +1,14 @@
 use crate::ui::Model;
 use std::path::Path;
 use termusiclib::config::SharedTuiSettings;
-use termusiclib::library_db::SearchCriteria;
+use termusiclib::library_db::const_unknown::{UNKNOWN_ARTIST, UNKNOWN_FILE, UNKNOWN_TITLE};
+use termusiclib::library_db::{Indexable, SearchCriteria};
 use termusiclib::types::{DBMsg, Id, Msg};
 use termusiclib::utils::{is_playlist, playlist_get_vec};
 use tui_realm_stdlib::List;
 use tuirealm::command::{Cmd, CmdResult, Direction, Position};
 use tuirealm::props::Borders;
-use tuirealm::props::{Alignment, BorderType, TableBuilder, TextSpan};
+use tuirealm::props::{Alignment, BorderType, Table, TableBuilder, TextSpan};
 use tuirealm::{
     event::{Key, KeyEvent, KeyModifiers, NoUserEvent},
     AttrValue, Attribute, Component, Event, MockComponent, State, StateValue,
@@ -68,7 +69,7 @@ impl Component<Msg, NoUserEvent> for DBListCriteria {
     fn on(&mut self, ev: Event<NoUserEvent>) -> Option<Msg> {
         let config = self.config.clone();
         let keys = &config.read().settings.keys;
-        let _cmd_result = match ev {
+        let cmd_result = match ev {
             Event::Keyboard(KeyEvent {
                 code: Key::Down,
                 modifiers: KeyModifiers::NONE,
@@ -145,7 +146,10 @@ impl Component<Msg, NoUserEvent> for DBListCriteria {
             }
             _ => CmdResult::None,
         };
-        Some(Msg::None)
+        match cmd_result {
+            CmdResult::None => None,
+            _ => Some(Msg::ForceRedraw),
+        }
     }
 }
 
@@ -196,7 +200,7 @@ impl Component<Msg, NoUserEvent> for DBListSearchResult {
     fn on(&mut self, ev: Event<NoUserEvent>) -> Option<Msg> {
         let config = self.config.clone();
         let keys = &config.read().settings.keys;
-        let _cmd_result = match ev {
+        let cmd_result = match ev {
             Event::Keyboard(KeyEvent {
                 code: Key::Down,
                 modifiers: KeyModifiers::NONE,
@@ -286,7 +290,10 @@ impl Component<Msg, NoUserEvent> for DBListSearchResult {
 
             _ => CmdResult::None,
         };
-        Some(Msg::None)
+        match cmd_result {
+            CmdResult::None => None,
+            _ => Some(Msg::ForceRedraw),
+        }
     }
 }
 
@@ -337,7 +344,7 @@ impl Component<Msg, NoUserEvent> for DBListSearchTracks {
     fn on(&mut self, ev: Event<NoUserEvent>) -> Option<Msg> {
         let config = self.config.clone();
         let keys = &config.read().settings.keys;
-        let _cmd_result = match ev {
+        let cmd_result = match ev {
             Event::Keyboard(KeyEvent {
                 code: Key::Down,
                 modifiers: KeyModifiers::NONE,
@@ -412,7 +419,10 @@ impl Component<Msg, NoUserEvent> for DBListSearchTracks {
 
             _ => CmdResult::None,
         };
-        Some(Msg::None)
+        match cmd_result {
+            CmdResult::None => None,
+            _ => Some(Msg::ForceRedraw),
+        }
     }
 }
 
@@ -573,7 +583,6 @@ impl Model {
         self.app.active(&Id::DBListSearchTracks).ok();
     }
 
-    #[allow(unused)]
     pub fn database_reload(&mut self) {
         assert!(self
             .app
@@ -618,46 +627,73 @@ impl Model {
         self.database_sync_results();
     }
 
-    pub fn database_update_search(&mut self, input: &str) {
-        let mut table: TableBuilder = TableBuilder::default();
-        let mut idx = 0;
+    fn match_record<T: Indexable>(record: &T, search: &str) -> bool {
+        let artist_match: bool = if let Some(artist) = record.meta_artist() {
+            wildmatch::WildMatch::new(search).matches(&artist.to_lowercase())
+        } else {
+            false
+        };
+        let title_match: bool = if let Some(title) = record.meta_title() {
+            wildmatch::WildMatch::new(search).matches(&title.to_lowercase())
+        } else {
+            false
+        };
+        let album_match: bool = if let Some(album) = record.meta_album() {
+            wildmatch::WildMatch::new(search).matches(&album.to_lowercase())
+        } else {
+            false
+        };
+        artist_match || title_match || album_match
+    }
+
+    pub fn update_search<'a, T: Indexable>(
+        indexable_songs: &'a [T],
+        input: &'a str,
+    ) -> impl Iterator<Item = &'a T> {
         let search = format!("*{}*", input.to_lowercase());
+        indexable_songs
+            .iter()
+            .filter(move |&record| Model::match_record(record, &search))
+    }
+
+    pub fn build_table<T: Indexable, I: Iterator<Item = T>>(data: I) -> Table {
+        let mut peekable_data = data.peekable();
+        let mut table: TableBuilder = TableBuilder::default();
+        if peekable_data.peek().is_none() {
+            table.add_col(TextSpan::from("0"));
+            table.add_col(TextSpan::from("empty tracks from db/playlist"));
+            table.add_col(TextSpan::from(""));
+            return table.build();
+        }
+
+        for (idx, record) in peekable_data.enumerate() {
+            if idx > 0 {
+                table.add_row();
+            }
+
+            let duration =
+                termusiclib::track::Track::duration_formatted_short(&record.meta_duration());
+            let duration_string = format!("[{duration:^6.6}]");
+
+            table
+                .add_col(TextSpan::new(duration_string))
+                .add_col(
+                    TextSpan::new(record.meta_artist().unwrap_or(UNKNOWN_ARTIST))
+                        .fg(tuirealm::ratatui::style::Color::LightYellow),
+                )
+                .add_col(TextSpan::new(record.meta_title().unwrap_or(UNKNOWN_TITLE)).bold())
+                .add_col(TextSpan::new(record.meta_file().unwrap_or(UNKNOWN_FILE)));
+        }
+        table.build()
+    }
+
+    pub fn database_update_search(&mut self, input: &str) {
         let mut db_tracks = vec![];
         if let Ok(tracks) = self.db.get_all_records() {
             db_tracks.clone_from(&tracks);
-            for record in tracks {
-                if wildmatch::WildMatch::new(&search).matches(&record.artist.to_lowercase())
-                    | wildmatch::WildMatch::new(&search).matches(&record.title.to_lowercase())
-                {
-                    if idx > 0 {
-                        table.add_row();
-                    }
-
-                    let duration =
-                        termusiclib::track::Track::duration_formatted_short(&record.duration);
-                    let duration_string = format!("[{duration:^6.6}]");
-
-                    table
-                        .add_col(TextSpan::new(duration_string.as_str()))
-                        .add_col(
-                            TextSpan::new(record.artist)
-                                .fg(tuirealm::tui::style::Color::LightYellow),
-                        )
-                        .add_col(TextSpan::new(record.title).bold())
-                        .add_col(TextSpan::new(record.file));
-                    // .add_col(TextSpan::new(record.album().unwrap_or("Unknown Album")));
-                    idx += 1;
-                }
-            }
         }
 
-        if db_tracks.is_empty() {
-            table.add_col(TextSpan::from("0"));
-            table.add_col(TextSpan::from("empty tracks from db"));
-            table.add_col(TextSpan::from(""));
-        }
-        let table = table.build();
-
-        self.general_search_update_show(table);
+        let filtered_music = Model::update_search(&db_tracks, input);
+        self.general_search_update_show(Model::build_table(filtered_music));
     }
 }
