@@ -1,12 +1,12 @@
+use crate::ui::tui_cmd::TuiCmd;
 use crate::ui::{Id, LIMsg, Model, Msg, TEMsg, YSMsg};
 use crate::utils::get_pin_yin;
 use anyhow::{bail, Context, Result};
-use std::fs::{remove_dir_all, remove_file, rename};
+use std::fs::{remove_dir_all, remove_file, rename, DirEntry};
 use std::path::{Path, PathBuf};
 use termusiclib::config::v2::server::config_extra::ServerConfigVersionedDefaulted;
 use termusiclib::config::v2::server::ScanDepth;
 use termusiclib::config::SharedTuiSettings;
-use termusicplayback::PlayerCmd;
 use tui_realm_treeview::{Node, Tree, TreeView, TREE_CMD_CLOSE, TREE_CMD_OPEN, TREE_INITIAL_NODE};
 use tuirealm::command::{Cmd, CmdResult, Direction, Position};
 use tuirealm::event::{Key, KeyEvent, KeyModifiers, NoUserEvent};
@@ -15,13 +15,17 @@ use tuirealm::{AttrValue, Attribute, Component, Event, MockComponent, State, Sta
 
 #[derive(MockComponent)]
 pub struct MusicLibrary {
-    component: TreeView,
+    component: TreeView<String>,
     config: SharedTuiSettings,
     pub init: bool,
 }
 
 impl MusicLibrary {
-    pub fn new(tree: &Tree, initial_node: Option<String>, config: SharedTuiSettings) -> Self {
+    pub fn new(
+        tree: &Tree<String>,
+        initial_node: Option<String>,
+        config: SharedTuiSettings,
+    ) -> Self {
         // Preserve initial node if exists
         let initial_node = match initial_node {
             Some(id) if tree.root().query(&id).is_some() => id,
@@ -225,7 +229,8 @@ impl Component<Msg, NoUserEvent> for MusicLibrary {
             CmdResult::Submit(State::One(StateValue::String(node))) => {
                 Some(Msg::Library(LIMsg::TreeStepInto(node)))
             }
-            _ => Some(Msg::None),
+            CmdResult::None => None,
+            _ => Some(Msg::ForceRedraw),
         }
     }
 }
@@ -246,12 +251,12 @@ impl Model {
             .map(std::path::Path::to_path_buf)
     }
 
-    pub fn library_dir_tree(p: &Path, depth: ScanDepth) -> Node {
+    pub fn library_dir_tree(p: &Path, depth: ScanDepth) -> Node<String> {
         let name: String = match p.file_name() {
             None => "/".to_string(),
             Some(n) => n.to_string_lossy().into_owned(),
         };
-        let mut node: Node = Node::new(p.to_string_lossy().into_owned(), name);
+        let mut node: Node<String> = Node::new(p.to_string_lossy().into_owned(), name);
 
         let depth = match depth {
             ScanDepth::Limited(v) => v,
@@ -261,17 +266,16 @@ impl Model {
 
         if depth > 0 && p.is_dir() {
             if let Ok(paths) = std::fs::read_dir(p) {
-                let mut paths: Vec<_> = paths
+                let mut paths: Vec<(String, PathBuf)> = paths
                     .filter_map(std::result::Result::ok)
                     .filter(|p| !p.file_name().to_string_lossy().starts_with('.'))
+                    .map(|v| (get_pin_yin(&v.file_name().to_string_lossy()), v.path()))
                     .collect();
 
-                paths.sort_by_cached_key(|k| get_pin_yin(&k.file_name().to_string_lossy()));
+                paths.sort_by(|a, b| alphanumeric_sort::compare_str(&a.0, &b.0));
+
                 for p in paths {
-                    node.add_child(Self::library_dir_tree(
-                        p.path().as_path(),
-                        ScanDepth::Limited(depth - 1),
-                    ));
+                    node.add_child(Self::library_dir_tree(&p.1, ScanDepth::Limited(depth - 1)));
                 }
             }
         }
@@ -281,10 +285,14 @@ impl Model {
         let mut children: Vec<String> = vec![];
         if p.is_dir() {
             if let Ok(paths) = std::fs::read_dir(p) {
-                let mut paths: Vec<_> = paths.filter_map(std::result::Result::ok).collect();
+                let mut paths: Vec<(String, DirEntry)> = paths
+                    .filter_map(std::result::Result::ok)
+                    .map(|v| (get_pin_yin(&v.file_name().to_string_lossy()), v))
+                    .collect();
 
-                paths.sort_by_cached_key(|k| get_pin_yin(&k.file_name().to_string_lossy()));
-                for p in paths {
+                paths.sort_by(|a, b| alphanumeric_sort::compare_str(&a.0, &b.0));
+
+                for (_, p) in paths {
                     if !p.path().is_dir() {
                         children.push(String::from(p.path().to_string_lossy()));
                     }
@@ -334,7 +342,7 @@ impl Model {
                 .mount(
                     Id::Library,
                     Box::new(MusicLibrary::new(
-                        &self.library.tree.clone(),
+                        &self.library.tree,
                         current_node,
                         self.config_tui.clone(),
                     ),),
@@ -350,7 +358,7 @@ impl Model {
             .remount(
                 Id::Library,
                 Box::new(MusicLibrary::new(
-                    &self.library.tree.clone(),
+                    &self.library.tree,
                     current_node,
                     self.config_tui.clone(),
                 ),),
@@ -385,7 +393,7 @@ impl Model {
             if p.is_file() {
                 self.mount_confirm_radio();
             } else {
-                self.mount_confirm_input();
+                self.mount_confirm_input("You're about to delete the whole directory.");
             }
         }
     }
@@ -465,7 +473,7 @@ impl Model {
         let root = self.library.tree.root();
         let p: &Path = Path::new(root.id());
         let all_items = walkdir::WalkDir::new(p).follow_links(true);
-        let mut idx = 0;
+        let mut idx: usize = 0;
         let search = format!("*{}*", input.to_lowercase());
         let search = wildmatch::WildMatch::new(&search);
         for record in all_items.into_iter().filter_map(std::result::Result::ok) {
@@ -539,7 +547,7 @@ impl Model {
         drop(config_server);
 
         res.context("Error while saving config")?;
-        self.command(&PlayerCmd::ReloadConfig);
+        self.command(TuiCmd::ReloadConfig);
         Ok(())
     }
 
