@@ -1,16 +1,18 @@
 use crate::ui::model::TermusicLayout;
+use crate::ui::tui_cmd::TuiCmd;
 use crate::ui::Model;
 use anyhow::{anyhow, bail, Result};
 use rand::seq::SliceRandom;
 use std::borrow::Cow;
+use std::ffi::OsString;
 use std::path::Path;
 use termusiclib::config::SharedTuiSettings;
+use termusiclib::library_db::const_unknown::{UNKNOWN_ALBUM, UNKNOWN_ARTIST};
 use termusiclib::library_db::SearchCriteria;
 use termusiclib::library_db::TrackDB;
 use termusiclib::track::Track;
 use termusiclib::types::{GSMsg, Id, Msg, PLMsg};
 use termusiclib::utils::{filetype_supported, get_parent_folder, is_playlist, playlist_get_vec};
-use termusicplayback::PlayerCmd;
 
 use tui_realm_stdlib::Table;
 use tuirealm::props::Borders;
@@ -70,7 +72,7 @@ impl Component<Msg, NoUserEvent> for Playlist {
     fn on(&mut self, ev: Event<NoUserEvent>) -> Option<Msg> {
         let config = self.config.clone();
         let keys = &config.read().settings.keys;
-        let _cmd_result = match ev {
+        let cmd_result = match ev {
             Event::Keyboard(KeyEvent {
                 code: Key::Down,
                 modifiers: KeyModifiers::NONE,
@@ -120,7 +122,7 @@ impl Component<Msg, NoUserEvent> for Playlist {
                     State::One(StateValue::Usize(index_selected)) => {
                         return Some(Msg::Playlist(PLMsg::Delete(index_selected)))
                     }
-                    _ => return Some(Msg::None),
+                    _ => CmdResult::None,
                 }
             }
             Event::Keyboard(key) if key == keys.playlist_keys.delete_all.get() => {
@@ -157,7 +159,7 @@ impl Component<Msg, NoUserEvent> for Playlist {
                         self.perform(Cmd::Move(Direction::Down));
                         return Some(Msg::Playlist(PLMsg::SwapDown(index_selected)));
                     }
-                    _ => return Some(Msg::None),
+                    _ => CmdResult::None,
                 }
             }
             Event::Keyboard(key) if key == keys.playlist_keys.swap_up.get() => {
@@ -166,7 +168,7 @@ impl Component<Msg, NoUserEvent> for Playlist {
                         self.perform(Cmd::Move(Direction::Up));
                         return Some(Msg::Playlist(PLMsg::SwapUp(index_selected)));
                     }
-                    _ => return Some(Msg::None),
+                    _ => CmdResult::None,
                 }
             }
             Event::Keyboard(key) if key == keys.playlist_keys.add_random_album.get() => {
@@ -177,7 +179,10 @@ impl Component<Msg, NoUserEvent> for Playlist {
             }
             _ => CmdResult::None,
         };
-        Some(Msg::None)
+        match cmd_result {
+            CmdResult::None => None,
+            _ => Some(Msg::ForceRedraw),
+        }
     }
 }
 
@@ -254,14 +259,19 @@ impl Model {
         self.playlist_sync();
     }
 
+    /// Add a playlist (like m3u) to the playlist.
     fn playlist_add_playlist(&mut self, current_node: &str) -> Result<()> {
         let vec = playlist_get_vec(current_node)?;
-        self.playlist.add_playlist(&vec)?;
+        // "add_playlist" will still add all possible things, but return errors for which did not work
+        // so sync still needs to happen
+        let res = self.playlist.add_playlist(&vec);
         self.player_sync_playlist()?;
         self.playlist_sync();
+        res?;
         Ok(())
     }
 
+    /// Add a podcast episode to the playlist.
     pub fn playlist_add_episode(&mut self, episode_index: usize) -> Result<()> {
         if self.podcast.podcasts.is_empty() {
             return Ok(());
@@ -281,6 +291,9 @@ impl Model {
         Ok(())
     }
 
+    /// Add the `current_node`, regardless if it is a Track, dir, playlist, etc.
+    ///
+    /// See [`Model::playlist_add_episode`] for podcast episode adding
     pub fn playlist_add(&mut self, current_node: &str) -> Result<()> {
         let p: &Path = Path::new(&current_node);
         if !p.exists() {
@@ -288,9 +301,12 @@ impl Model {
         }
         if p.is_dir() {
             let new_items_vec = Self::library_dir_children(p);
-            self.playlist.add_playlist(&new_items_vec)?;
+            // "add_playlist" will still add all possible things, but return errors for which did not work
+            // so sync still needs to happen
+            let res = self.playlist.add_playlist(&new_items_vec);
             self.player_sync_playlist()?;
             self.playlist_sync();
+            res?;
             return Ok(());
         }
         self.playlist_add_item(current_node)?;
@@ -298,27 +314,33 @@ impl Model {
         Ok(())
     }
 
+    /// Add a Track or a Playlist to the playlist
     fn playlist_add_item(&mut self, current_node: &str) -> Result<()> {
         if is_playlist(current_node) {
             self.playlist_add_playlist(current_node)?;
             return Ok(());
         }
-        self.playlist.add_playlist(&[current_node])?;
+        self.playlist.add_track(&current_node)?;
         self.player_sync_playlist()?;
         Ok(())
     }
 
+    /// Add [`TrackDB`] to the playlist
     pub fn playlist_add_all_from_db(&mut self, vec: &[TrackDB]) {
         let vec2: Vec<&str> = vec.iter().map(|f| f.file.as_str()).collect();
-        if let Err(e) = self.playlist.add_playlist(&vec2) {
-            self.mount_error_popup(e.context("add all to playlist from database"));
-        }
+        // "add_playlist" will still add all possible things, but return errors for which did not work
+        // so sync still needs to happen
+        let res = self.playlist.add_playlist(&vec2);
         if let Err(e) = self.player_sync_playlist() {
             self.mount_error_popup(e.context("player sync playlist"));
         }
         self.playlist_sync();
+        if let Err(e) = res {
+            self.mount_error_popup(anyhow!(e).context("add all to playlist from database"));
+        }
     }
 
+    /// Add random album(s) from the database to the playlist
     pub fn playlist_add_random_album(&mut self) {
         let playlist_select_random_album_quantity = self
             .config_server
@@ -331,6 +353,7 @@ impl Model {
         self.playlist_add_all_from_db(&vec);
     }
 
+    /// Add random tracks from the database to the playlist
     pub fn playlist_add_random_tracks(&mut self) {
         let playlist_select_random_track_quantity = self
             .config_server
@@ -409,9 +432,9 @@ impl Model {
 
             let noname_string = "No Name".to_string();
             let name = record.name().unwrap_or(&noname_string);
-            let artist = record.artist().unwrap_or(name);
-            let mut title: Cow<'_, str> = record.title().unwrap_or("Unknown Title").into();
-            let album = record.album().unwrap_or("Unknown Album");
+            let artist = record.artist().unwrap_or(UNKNOWN_ARTIST);
+            let mut title: Cow<'_, str> = record.title().unwrap_or(name).into();
+            let album = record.album().unwrap_or(UNKNOWN_ALBUM);
 
             // TODO: is there maybe a better option to do this on-demand instead of the whole playlist; like on draw-time?
             if idx == self.playlist.get_current_track_index() {
@@ -430,7 +453,7 @@ impl Model {
 
             table
                 .add_col(TextSpan::new(duration_string.as_str()))
-                .add_col(TextSpan::new(artist).fg(tuirealm::tui::style::Color::LightYellow))
+                .add_col(TextSpan::new(artist).fg(tuirealm::ratatui::style::Color::LightYellow))
                 .add_col(TextSpan::new(title).bold())
                 .add_col(TextSpan::new(album));
         }
@@ -518,49 +541,12 @@ impl Model {
         if let Err(e) = self.player_sync_playlist() {
             self.mount_error_popup(e.context("player sync playlist"));
         }
-        self.command(&PlayerCmd::PlaySelected);
+        self.command(TuiCmd::PlaySelected);
     }
 
     pub fn playlist_update_search(&mut self, input: &str) {
-        let mut table: TableBuilder = TableBuilder::default();
-        let mut idx = 0;
-        let search = format!("*{}*", input.to_lowercase());
-        for record in self.playlist.tracks() {
-            let artist = record.artist().unwrap_or("Unknown artist");
-            let title = record.title().unwrap_or("Unknown title");
-            if wildmatch::WildMatch::new(&search).matches(&artist.to_lowercase())
-                | wildmatch::WildMatch::new(&search).matches(&title.to_lowercase())
-            {
-                if idx > 0 {
-                    table.add_row();
-                }
-
-                let duration = record.duration_formatted().to_string();
-                let duration_string = format!("[{duration:^6.6}]");
-
-                let noname_string = "No Name".to_string();
-                let name = record.name().unwrap_or(&noname_string);
-                let artist = record.artist().unwrap_or(name);
-                let title = record.title().unwrap_or("Unknown Title");
-                let file_name = record.file().unwrap_or("no file");
-
-                table
-                    .add_col(TextSpan::new(duration_string.as_str()))
-                    .add_col(TextSpan::new(artist).fg(tuirealm::tui::style::Color::LightYellow))
-                    .add_col(TextSpan::new(title).bold())
-                    .add_col(TextSpan::new(file_name));
-                // .add_col(TextSpan::new(record.album().unwrap_or("Unknown Album")));
-                idx += 1;
-            }
-        }
-        if self.playlist.is_empty() {
-            table.add_col(TextSpan::from("0"));
-            table.add_col(TextSpan::from("empty playlist"));
-            table.add_col(TextSpan::from(""));
-        }
-        let table = table.build();
-
-        self.general_search_update_show(table);
+        let filtered_music = Model::update_search(self.playlist.tracks(), input);
+        self.general_search_update_show(Model::build_table(filtered_music));
     }
 
     pub fn playlist_locate(&mut self, index: usize) {
@@ -620,30 +606,35 @@ impl Model {
         result
     }
 
+    /// Save the current playlist as m3u with the given `filename`
     pub fn playlist_save_m3u_before(&mut self, filename: &str) -> Result<()> {
         let current_node: String = match self.app.state(&Id::Library).ok().unwrap() {
             State::One(StateValue::String(id)) => id,
             _ => bail!("Invalid node selected in library"),
         };
 
-        let parent_folder = get_parent_folder(&current_node);
+        let path_m3u = {
+            let mut parent_folder = get_parent_folder(Path::new(&current_node)).to_path_buf();
+            let mut filename = OsString::from(filename);
+            filename.push(".m3u");
+            parent_folder.push(filename);
 
-        let full_filename = format!("{parent_folder}/{filename}.m3u");
-
-        let path_m3u = Path::new(&full_filename);
+            parent_folder
+        };
 
         if path_m3u.exists() {
-            self.mount_save_playlist_confirm(&full_filename);
+            self.mount_save_playlist_confirm(&path_m3u.to_string_lossy());
             return Ok(());
         }
 
-        self.playlist_save_m3u(&full_filename)
+        self.playlist_save_m3u(&path_m3u)
     }
 
-    pub fn playlist_save_m3u(&mut self, filename: &str) -> Result<()> {
+    /// Save the current playlist as m3u in the given full path
+    pub fn playlist_save_m3u(&mut self, filename: &Path) -> Result<()> {
         self.playlist.save_m3u(filename)?;
 
-        self.library_reload_with_node_focus(Some(filename));
+        self.library_reload_with_node_focus(Some(&filename.to_string_lossy()));
 
         Ok(())
     }
