@@ -25,9 +25,9 @@ use termusiclib::config::SharedTuiSettings;
 use tui_realm_stdlib::utils::get_block;
 use tuirealm::command::{Cmd, CmdResult};
 use tuirealm::event::{Key, KeyEvent, KeyModifiers, NoUserEvent};
-use tuirealm::props::{Alignment, Borders, Color, Style, TextModifiers};
-use tuirealm::tui::layout::Rect;
-use tuirealm::tui::widgets::{BorderType, Paragraph};
+use tuirealm::props::{Alignment, Borders, Color, PropPayload, PropValue, Style, TextModifiers};
+use tuirealm::ratatui::layout::Rect;
+use tuirealm::ratatui::widgets::{BorderType, Paragraph};
 use tuirealm::{
     AttrValue, Attribute, Component, Event, Frame, MockComponent, Props, State, StateValue,
 };
@@ -39,11 +39,10 @@ use crate::ui::{Model, Msg, TEMsg, TFMsg};
 #[derive(Default)]
 struct Counter {
     props: Props,
-    states: OwnStates,
 }
 
-#[allow(unused)]
 impl Counter {
+    #[allow(dead_code)]
     pub fn label<S>(mut self, label: S) -> Self
     where
         S: AsRef<str>,
@@ -55,8 +54,15 @@ impl Counter {
         self
     }
 
-    pub fn value(mut self, n: isize) -> Self {
-        self.attr(Attribute::Value, AttrValue::Number(n));
+    pub fn value(mut self, n: Option<usize>) -> Self {
+        if let Some(n) = n {
+            self.attr(
+                Attribute::Value,
+                AttrValue::Payload(PropPayload::One(PropValue::Usize(n))),
+            );
+        } else {
+            self.attr(Attribute::Value, AttrValue::Payload(PropPayload::None));
+        }
         self
     }
 
@@ -84,6 +90,17 @@ impl Counter {
         self.attr(Attribute::Borders, AttrValue::Borders(b));
         self
     }
+
+    pub fn get_state(&self) -> Option<usize> {
+        match self
+            .props
+            .get(Attribute::Value)
+            .map(|v| v.unwrap_payload())?
+        {
+            PropPayload::One(PropValue::Usize(v)) => Some(v),
+            _ => None,
+        }
+    }
 }
 
 impl MockComponent for Counter {
@@ -91,11 +108,12 @@ impl MockComponent for Counter {
         // Check if visible
         if self.props.get_or(Attribute::Display, AttrValue::Flag(true)) == AttrValue::Flag(true) {
             // Get properties
-            let value = self
-                .props
-                .get_or(Attribute::Value, AttrValue::Number(99))
-                .unwrap_number();
-            let text = format!("Delete ({value})");
+            let value = self.get_state();
+            let text = if let Some(value) = value {
+                format!("Delete Selected ({value})")
+            } else {
+                "Delete Selected (-)".to_string()
+            };
 
             let alignment = self
                 .props
@@ -160,7 +178,11 @@ impl MockComponent for Counter {
     }
 
     fn state(&self) -> State {
-        State::One(StateValue::Isize(self.states.counter))
+        let Some(state) = self.get_state() else {
+            return State::None;
+        };
+
+        State::One(StateValue::Usize(state))
     }
 
     fn perform(&mut self, cmd: Cmd) -> CmdResult {
@@ -174,23 +196,6 @@ impl MockComponent for Counter {
     }
 }
 
-#[derive(Default)]
-struct OwnStates {
-    counter: isize,
-}
-
-// impl Default for OwnStates {
-//     fn default() -> Self {
-//         Self { counter: 0 }
-//     }
-// }
-#[allow(unused)]
-impl OwnStates {
-    fn incr(&mut self) {
-        self.counter += 1;
-    }
-}
-
 // -- Counter components
 
 #[derive(MockComponent)]
@@ -200,7 +205,7 @@ pub struct TECounterDelete {
 }
 
 impl TECounterDelete {
-    pub fn new(initial_value: isize, config: SharedTuiSettings) -> Self {
+    pub fn new(initial_value: Option<usize>, config: SharedTuiSettings) -> Self {
         let component = {
             let config = config.read();
             Counter::default()
@@ -283,7 +288,7 @@ impl Model {
                 song.set_lyric_selected_index(song.lyric_selected_index() - 1);
             }
             match song.save_tag() {
-                Ok(()) => self.init_by_song(&song),
+                Ok(()) => self.init_by_song(song),
                 Err(e) => {
                     self.mount_error_popup(e);
                 }
