@@ -28,13 +28,13 @@ use crate::ui::components::{
 };
 use crate::ui::model::Model;
 use crate::ui::utils::{draw_area_in_absolute, draw_area_top_right_absolute};
-use std::convert::TryFrom;
+use anyhow::Result;
 use std::path::Path;
 use termusiclib::track::Track;
 use termusiclib::types::{Id, IdTagEditor};
 use tuirealm::props::{Alignment, AttrValue, Attribute, PropPayload, PropValue, TextSpan};
-use tuirealm::tui::layout::{Constraint, Direction, Layout};
-use tuirealm::tui::widgets::Clear;
+use tuirealm::ratatui::layout::{Constraint, Direction, Layout};
+use tuirealm::ratatui::widgets::Clear;
 use tuirealm::State;
 
 impl Model {
@@ -49,7 +49,7 @@ impl Model {
                         _ => 8,
                     };
                 if self.app.mounted(&Id::TagEditor(IdTagEditor::LabelHint)) {
-                    f.render_widget(Clear, f.size());
+                    f.render_widget(Clear, f.area());
                     let chunks_main = Layout::default()
                         .direction(Direction::Vertical)
                         .margin(0)
@@ -64,7 +64,7 @@ impl Model {
                             ]
                             .as_ref(),
                         )
-                        .split(f.size());
+                        .split(f.area());
 
                     let chunks_row1 = Layout::default()
                         .direction(Direction::Horizontal)
@@ -158,12 +158,12 @@ impl Model {
                     );
 
                     if self.app.mounted(&Id::MessagePopup) {
-                        let popup = draw_area_top_right_absolute(f.size(), 25, 4);
+                        let popup = draw_area_top_right_absolute(f.area(), 25, 4);
                         f.render_widget(Clear, popup);
                         self.app.view(&Id::MessagePopup, f, popup);
                     }
                     if self.app.mounted(&Id::ErrorPopup) {
-                        let popup = draw_area_in_absolute(f.size(), 50, 4);
+                        let popup = draw_area_in_absolute(f.area(), 50, 4);
                         f.render_widget(Clear, popup);
                         self.app.view(&Id::ErrorPopup, f, popup);
                     }
@@ -172,6 +172,65 @@ impl Model {
             .expect("Expected to draw without error");
     }
 
+    /// Mount / Remount the Tag Editor
+    fn remount_tageditor(&mut self) -> Result<()> {
+        self.app.remount(
+            Id::Label,
+            Box::new(TEFooter::new(&self.config_tui.read())),
+            Vec::new(),
+        )?;
+        self.app.remount(
+            Id::TagEditor(IdTagEditor::LabelHint),
+            Box::new(LabelGeneric::new(
+                &self.config_tui.read(),
+                "Press <ENTER> to search:",
+            )),
+            Vec::new(),
+        )?;
+        self.app.remount(
+            Id::TagEditor(IdTagEditor::InputArtist),
+            Box::new(TEInputArtist::new(self.config_tui.clone())),
+            Vec::new(),
+        )?;
+        self.app.remount(
+            Id::TagEditor(IdTagEditor::InputTitle),
+            Box::new(TEInputTitle::new(self.config_tui.clone())),
+            Vec::new(),
+        )?;
+        self.app.remount(
+            Id::TagEditor(IdTagEditor::InputAlbum),
+            Box::new(TEInputAlbum::new(self.config_tui.clone())),
+            Vec::new(),
+        )?;
+        self.app.remount(
+            Id::TagEditor(IdTagEditor::InputGenre),
+            Box::new(TEInputGenre::new(self.config_tui.clone())),
+            Vec::new(),
+        )?;
+        self.app.remount(
+            Id::TagEditor(IdTagEditor::TableLyricOptions),
+            Box::new(TETableLyricOptions::new(self.config_tui.clone())),
+            Vec::new(),
+        )?;
+        self.app.remount(
+            Id::TagEditor(IdTagEditor::SelectLyric),
+            Box::new(TESelectLyric::new(self.config_tui.clone())),
+            Vec::new(),
+        )?;
+        self.app.remount(
+            Id::TagEditor(IdTagEditor::CounterDelete),
+            Box::new(TECounterDelete::new(None, self.config_tui.clone())),
+            Vec::new(),
+        )?;
+        self.app.remount(
+            Id::TagEditor(IdTagEditor::TextareaLyric),
+            Box::new(TETextareaLyric::new(self.config_tui.clone())),
+            Vec::new(),
+        )?;
+
+        Ok(())
+    }
+
     #[allow(clippy::too_many_lines)]
     pub fn mount_tageditor(&mut self, node_id: &str) {
         let node_path: &Path = Path::new(node_id);
@@ -188,137 +247,50 @@ impl Model {
             }
         };
 
-        assert!(self
-            .app
-            .remount(
-                Id::Label,
-                Box::new(TEFooter::new(&self.config_tui.read())),
-                Vec::default(),
-            )
-            .is_ok());
-        assert!(self
-            .app
-            .remount(
-                Id::TagEditor(IdTagEditor::LabelHint),
-                Box::new(LabelGeneric::new(
-                    &self.config_tui.read(),
-                    "Press <ENTER> to search:"
-                )),
-                Vec::new()
-            )
-            .is_ok());
-        assert!(self
-            .app
-            .remount(
-                Id::TagEditor(IdTagEditor::InputArtist),
-                Box::new(TEInputArtist::new(self.config_tui.clone())),
-                Vec::new()
-            )
-            .is_ok());
-        assert!(self
-            .app
-            .remount(
-                Id::TagEditor(IdTagEditor::InputTitle),
-                Box::new(TEInputTitle::new(self.config_tui.clone())),
-                Vec::new()
-            )
-            .is_ok());
-        assert!(self
-            .app
-            .remount(
-                Id::TagEditor(IdTagEditor::InputAlbum),
-                Box::new(TEInputAlbum::new(self.config_tui.clone())),
-                Vec::new()
-            )
-            .is_ok());
-        assert!(self
-            .app
-            .remount(
-                Id::TagEditor(IdTagEditor::InputGenre),
-                Box::new(TEInputGenre::new(self.config_tui.clone())),
-                Vec::new()
-            )
-            .is_ok());
-        assert!(self
-            .app
-            .remount(
-                Id::TagEditor(IdTagEditor::TableLyricOptions),
-                Box::new(TETableLyricOptions::new(self.config_tui.clone())),
-                Vec::new()
-            )
-            .is_ok());
-        assert!(self
-            .app
-            .remount(
-                Id::TagEditor(IdTagEditor::SelectLyric),
-                Box::new(TESelectLyric::new(self.config_tui.clone())),
-                Vec::new()
-            )
-            .is_ok());
-        assert!(self
-            .app
-            .remount(
-                Id::TagEditor(IdTagEditor::CounterDelete),
-                Box::new(TECounterDelete::new(5, self.config_tui.clone())),
-                Vec::new()
-            )
-            .is_ok());
-        assert!(self
-            .app
-            .remount(
-                Id::TagEditor(IdTagEditor::TextareaLyric),
-                Box::new(TETextareaLyric::new(self.config_tui.clone())),
-                Vec::new()
-            )
-            .is_ok());
+        self.remount_tageditor().unwrap();
 
         self.app
             .active(&Id::TagEditor(IdTagEditor::InputArtist))
             .ok();
-        self.init_by_song(&track);
+        self.init_by_song(track);
 
         if let Err(err) = self.update_photo() {
             self.mount_error_popup(err.context("update_photo"));
         }
     }
 
+    /// Unmount the Tag Editor
+    fn umount_tageditor_inner(&mut self) -> Result<()> {
+        self.app.umount(&Id::TagEditor(IdTagEditor::LabelHint))?;
+        self.app.umount(&Id::TagEditor(IdTagEditor::InputArtist))?;
+        self.app.umount(&Id::TagEditor(IdTagEditor::InputTitle))?;
+        self.app.umount(&Id::TagEditor(IdTagEditor::InputAlbum))?;
+        self.app.umount(&Id::TagEditor(IdTagEditor::InputGenre))?;
+        self.app
+            .umount(&Id::TagEditor(IdTagEditor::TableLyricOptions))?;
+        self.app.umount(&Id::TagEditor(IdTagEditor::SelectLyric))?;
+        self.app
+            .umount(&Id::TagEditor(IdTagEditor::CounterDelete))?;
+        self.app
+            .umount(&Id::TagEditor(IdTagEditor::TextareaLyric))?;
+
+        Ok(())
+    }
+
     pub fn umount_tageditor(&mut self) {
         self.mount_label_help();
-        self.app.umount(&Id::TagEditor(IdTagEditor::LabelHint)).ok();
-        self.app
-            .umount(&Id::TagEditor(IdTagEditor::InputArtist))
-            .ok();
-        self.app
-            .umount(&Id::TagEditor(IdTagEditor::InputTitle))
-            .ok();
-        self.app
-            .umount(&Id::TagEditor(IdTagEditor::InputAlbum))
-            .ok();
-        self.app
-            .umount(&Id::TagEditor(IdTagEditor::InputGenre))
-            .ok();
-        // self.app.umount(&Id::TagEditor(IdTagEditor::RadioTag)).ok();
-        self.app
-            .umount(&Id::TagEditor(IdTagEditor::TableLyricOptions))
-            .ok();
-        self.app
-            .umount(&Id::TagEditor(IdTagEditor::SelectLyric))
-            .ok();
-        self.app
-            .umount(&Id::TagEditor(IdTagEditor::CounterDelete))
-            .ok();
-        self.app
-            .umount(&Id::TagEditor(IdTagEditor::TextareaLyric))
-            .ok();
+        self.umount_tageditor_inner().unwrap();
         if let Err(e) = self.update_photo() {
             self.mount_error_popup(e.context("update_photo"));
         }
     }
 
-    // initialize the value in tageditor based on info from Song
+    /// Set the Lyric section of the tag-editor to the Lyrics based on the provided Track
     #[allow(clippy::too_many_lines)]
-    pub fn init_by_song(&mut self, s: &Track) {
-        self.tageditor_song = Some(s.clone());
+    pub fn init_by_song(&mut self, s: Track) {
+        self.tageditor_song = Some(s);
+        // Unwrap safe as we literally just assigned it
+        let s = self.tageditor_song.as_ref().unwrap();
         if let Some(artist) = s.artist() {
             assert!(self
                 .app
@@ -363,57 +335,71 @@ impl Model {
                 .is_ok());
         }
 
-        if s.lyric_frames_is_empty() {
+        let Some(lf) = s.lyric_frames() else {
             self.init_by_song_no_lyric();
             return;
-        }
+        };
 
-        let mut vec_lang: Vec<String> = vec![];
-        if let Some(lf) = s.lyric_frames() {
-            for l in lf {
-                vec_lang.push(l.description.clone());
-            }
-        }
-        vec_lang.sort();
+        let vec_lang: Vec<PropValue> = lf
+            .into_iter()
+            .enumerate()
+            .map(|(idx, v)| {
+                let val = v.description;
+                // match idx with Delete counter
+                let idx = idx + 1;
+                if val.is_empty() {
+                    format!("{idx} - {}", v.lang)
+                } else {
+                    format!("{idx} - {val}")
+                }
+            })
+            .map(PropValue::Str)
+            .collect();
+
+        let selected_index = s.lyric_selected_index();
+        // get access to "Lyric" instance for text and modified description for display
+        let (Some(vec_lang_selected), Some(selected_desc)) =
+            (s.lyric_selected(), vec_lang.get(selected_index))
+        else {
+            // this should not happen as if it is Some above, there should be at least one entry in the vec.
+            self.init_by_song_no_lyric();
+            return;
+        };
+        // TODO: replace with tui-realm "as_str" once available
+        let PropValue::Str(selected_desc) = selected_desc else {
+            unreachable!()
+        };
+        let selected_description = format!("Lyrics for {selected_desc}");
+        let selected_index_display = selected_index + 1;
+
+        let vec_lyric = vec_lang_selected
+            .text
+            .lines()
+            .map(|line| PropValue::TextSpan(TextSpan::from(line.trim())))
+            .collect();
 
         assert!(self
             .app
             .attr(
                 &Id::TagEditor(IdTagEditor::SelectLyric),
                 Attribute::Content,
-                AttrValue::Payload(PropPayload::Vec(
-                    vec_lang
-                        .iter()
-                        .map(|x| PropValue::Str((*x).to_string()))
-                        .collect(),
-                )),
+                AttrValue::Payload(PropPayload::Vec(vec_lang)),
+            )
+            .is_ok());
+        assert!(self
+            .app
+            .attr(
+                &Id::TagEditor(IdTagEditor::CounterDelete),
+                Attribute::Value,
+                AttrValue::Payload(PropPayload::One(PropValue::Usize(selected_index_display))),
             )
             .is_ok());
-        if let Ok(vec_lang_len_isize) = isize::try_from(vec_lang.len()) {
-            assert!(self
-                .app
-                .attr(
-                    &Id::TagEditor(IdTagEditor::CounterDelete),
-                    Attribute::Value,
-                    AttrValue::Number(vec_lang_len_isize),
-                )
-                .is_ok());
-        }
-        let mut vec_lyric: Vec<TextSpan> = vec![];
-        if let Some(f) = s.lyric_selected() {
-            for line in f.text.split('\n') {
-                vec_lyric.push(TextSpan::from(line));
-            }
-        }
         assert!(self
             .app
             .attr(
                 &Id::TagEditor(IdTagEditor::TextareaLyric),
                 Attribute::Title,
-                AttrValue::Title((
-                    format!("{} Lyrics", vec_lang[s.lyric_selected_index()]),
-                    Alignment::Left
-                ))
+                AttrValue::Title((selected_description, Alignment::Left))
             )
             .is_ok());
 
@@ -422,13 +408,12 @@ impl Model {
             .attr(
                 &Id::TagEditor(IdTagEditor::TextareaLyric),
                 Attribute::Text,
-                AttrValue::Payload(PropPayload::Vec(
-                    vec_lyric.iter().cloned().map(PropValue::TextSpan).collect()
-                ))
+                AttrValue::Payload(PropPayload::Vec(vec_lyric))
             )
             .is_ok());
     }
 
+    /// Set the Lyric section of the tag-editor to "No Lyrics" (ie clear state)
     fn init_by_song_no_lyric(&mut self) {
         assert!(self
             .app
@@ -448,7 +433,7 @@ impl Model {
             .attr(
                 &Id::TagEditor(IdTagEditor::CounterDelete),
                 Attribute::Value,
-                AttrValue::Number(0),
+                AttrValue::Payload(PropPayload::None),
             )
             .is_ok());
 
@@ -457,7 +442,7 @@ impl Model {
             .attr(
                 &Id::TagEditor(IdTagEditor::TextareaLyric),
                 Attribute::Title,
-                AttrValue::Title(("Empty Lyric".to_string(), Alignment::Left))
+                AttrValue::Title(("Empty Lyrics".to_string(), Alignment::Left))
             )
             .is_ok());
         assert!(self
