@@ -1,14 +1,4 @@
-use crate::ui::{model::TermusicLayout, Model};
-use anyhow::anyhow;
-use std::thread::{self, sleep};
-use std::time::Duration;
-use termusiclib::library_db::SearchCriteria;
-use termusiclib::track::MediaType;
-use termusiclib::types::{
-    DBMsg, DLMsg, GSMsg, Id, IdTagEditor, LIMsg, LyricMsg, Msg, PCMsg, PLMsg, XYWHMsg, YSMsg,
-};
-use termusicplayback::PlayerCmd;
-/**
+/*!
  * MIT License
  *
  * termusic - Copyright (C) 2021 Larry Hao
@@ -31,149 +21,158 @@ use termusicplayback::PlayerCmd;
  * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  * SOFTWARE.
  */
-// use termusicplayback::{PlayerMsg, PlayerTrait};
+
+use crate::ui::tui_cmd::TuiCmd;
+use crate::ui::{model::TermusicLayout, Model};
+use anyhow::anyhow;
+use std::path::Path;
+use std::time::Duration;
+use termusiclib::library_db::SearchCriteria;
+use termusiclib::track::MediaType;
+use termusiclib::types::{
+    DBMsg, DLMsg, GSMsg, Id, IdTagEditor, LIMsg, LyricMsg, Msg, PCMsg, PLMsg, XYWHMsg, YSMsg,
+};
+use tokio::runtime::Handle;
+use tokio::time::sleep;
 use tuirealm::props::{AttrValue, Attribute};
 use tuirealm::Update;
 
 impl Update<Msg> for Model {
     #[allow(clippy::too_many_lines)]
     fn update(&mut self, msg: Option<Msg>) -> Option<Msg> {
-        // match msg.unwrap_or(Msg::None) {
-        if let Some(msg) = msg {
-            // Set redraw
-            self.redraw = true;
-            // Match message
-            match msg {
-                Msg::ConfigEditor(m) => self.update_config_editor(m),
-                Msg::DataBase(m) => self.update_database_list(&m),
+        let msg = msg?;
+        // Set redraw
+        self.redraw = true;
+        // Match message
+        match msg {
+            Msg::ConfigEditor(m) => self.update_config_editor(m),
+            Msg::DataBase(m) => self.update_database_list(&m),
 
-                Msg::DeleteConfirmShow
-                | Msg::DeleteConfirmCloseCancel
-                | Msg::DeleteConfirmCloseOk => self.update_delete_confirmation(&msg),
-
-                Msg::ErrorPopupClose => {
-                    if self.app.mounted(&Id::ErrorPopup) {
-                        self.umount_error_popup();
-                    }
-                    None
-                }
-                Msg::QuitPopupShow => {
-                    if self.config_tui.read().settings.behavior.confirm_quit {
-                        self.mount_quit_popup();
-                    } else {
-                        self.quit = true;
-                    }
-                    None
-                }
-                Msg::QuitPopupCloseCancel => {
-                    self.app.umount(&Id::QuitPopup).ok();
-                    None
-                }
-                Msg::QuitPopupCloseOk => {
-                    self.quit = true;
-                    None
-                }
-                Msg::Library(m) => {
-                    self.update_library(&m);
-                    None
-                }
-                Msg::GeneralSearch(m) => {
-                    self.update_general_search(&m);
-                    None
-                }
-                Msg::Playlist(m) => {
-                    self.update_playlist(&m);
-                    None
-                }
-
-                Msg::PlayerTogglePause
-                | Msg::PlayerToggleGapless
-                | Msg::PlayerSpeedUp
-                | Msg::PlayerSpeedDown
-                | Msg::PlayerVolumeUp
-                | Msg::PlayerVolumeDown
-                | Msg::PlayerSeekForward
-                | Msg::PlayerSeekBackward => self.update_player(&msg),
-
-                Msg::HelpPopupShow => {
-                    self.mount_help_popup();
-                    None
-                }
-                Msg::HelpPopupClose => {
-                    if self.app.mounted(&Id::HelpPopup) {
-                        self.app.umount(&Id::HelpPopup).ok();
-                    }
-                    self.update_photo().ok();
-                    None
-                }
-                Msg::YoutubeSearch(m) => {
-                    self.update_youtube_search(&m);
-                    None
-                }
-                Msg::LyricCycle => {
-                    self.lyric_cycle();
-                    None
-                }
-                Msg::LyricAdjustDelay(offset) => {
-                    self.lyric_adjust_delay(offset);
-                    None
-                }
-                Msg::TagEditor(m) => {
-                    self.update_tageditor(&m);
-                    None
-                }
-                Msg::UpdatePhoto => {
-                    if let Err(e) = self.update_photo() {
-                        self.mount_error_popup(e.context("update_photo"));
-                    }
-                    None
-                }
-                Msg::LayoutDataBase | Msg::LayoutTreeView | Msg::LayoutPodCast => {
-                    self.update_layout(&msg)
-                }
-
-                Msg::None => None,
-                Msg::SavePlaylistPopupShow => {
-                    if let Err(e) = self.mount_save_playlist() {
-                        self.mount_error_popup(e.context("mount save playlist"));
-                    }
-                    None
-                }
-                Msg::SavePlaylistPopupCloseCancel => {
-                    self.umount_save_playlist();
-                    None
-                }
-                Msg::SavePlaylistPopupCloseOk(filename) => {
-                    self.umount_save_playlist();
-                    if let Err(e) = self.playlist_save_m3u_before(&filename) {
-                        self.mount_error_popup(e.context("save m3u playlist before"));
-                    }
-                    None
-                }
-                Msg::SavePlaylistPopupUpdate(filename) => {
-                    if let Err(e) = self.remount_save_playlist_label(&filename) {
-                        self.mount_error_popup(e.context("remount save playlist label"));
-                    }
-                    None
-                }
-                Msg::SavePlaylistConfirmCloseCancel => {
-                    self.umount_save_playlist_confirm();
-                    None
-                }
-                Msg::SavePlaylistConfirmCloseOk(filename) => {
-                    if let Err(e) = self.playlist_save_m3u(&filename) {
-                        self.mount_error_popup(e.context("save m3u playlist"));
-                    }
-                    self.umount_save_playlist_confirm();
-                    None
-                }
-                Msg::Podcast(m) => self.update_podcast(&m),
-                Msg::LyricMessage(m) => self.update_lyric_textarea(m),
-                Msg::Download(m) => self.update_download_msg(&m),
-                Msg::Xywh(m) => self.update_xywh_msg(m),
+            Msg::DeleteConfirmShow | Msg::DeleteConfirmCloseCancel | Msg::DeleteConfirmCloseOk => {
+                self.update_delete_confirmation(&msg)
             }
-        } else {
-            None
+
+            Msg::ErrorPopupClose => {
+                if self.app.mounted(&Id::ErrorPopup) {
+                    self.umount_error_popup();
+                }
+                None
+            }
+            Msg::QuitPopupShow => {
+                if self.config_tui.read().settings.behavior.confirm_quit {
+                    self.mount_quit_popup();
+                } else {
+                    self.quit = true;
+                }
+                None
+            }
+            Msg::QuitPopupCloseCancel => {
+                self.app.umount(&Id::QuitPopup).ok();
+                None
+            }
+            Msg::QuitPopupCloseOk => {
+                self.quit = true;
+                None
+            }
+            Msg::Library(m) => {
+                self.update_library(&m);
+                None
+            }
+            Msg::GeneralSearch(m) => {
+                self.update_general_search(&m);
+                None
+            }
+            Msg::Playlist(m) => {
+                self.update_playlist(&m);
+                None
+            }
+
+            Msg::PlayerTogglePause
+            | Msg::PlayerToggleGapless
+            | Msg::PlayerSpeedUp
+            | Msg::PlayerSpeedDown
+            | Msg::PlayerVolumeUp
+            | Msg::PlayerVolumeDown
+            | Msg::PlayerSeekForward
+            | Msg::PlayerSeekBackward => self.update_player(&msg),
+
+            Msg::HelpPopupShow => {
+                self.mount_help_popup();
+                None
+            }
+            Msg::HelpPopupClose => {
+                if self.app.mounted(&Id::HelpPopup) {
+                    self.app.umount(&Id::HelpPopup).ok();
+                }
+                self.update_photo().ok();
+                None
+            }
+            Msg::YoutubeSearch(m) => {
+                self.update_youtube_search(m);
+                None
+            }
+            Msg::LyricCycle => {
+                self.lyric_cycle();
+                None
+            }
+            Msg::LyricAdjustDelay(offset) => {
+                self.lyric_adjust_delay(offset);
+                None
+            }
+            Msg::TagEditor(m) => {
+                self.update_tageditor(&m);
+                None
+            }
+            Msg::UpdatePhoto => {
+                if let Err(e) = self.update_photo() {
+                    self.mount_error_popup(e.context("update_photo"));
+                }
+                None
+            }
+            Msg::LayoutDataBase | Msg::LayoutTreeView | Msg::LayoutPodCast => {
+                self.update_layout(&msg)
+            }
+
+            Msg::SavePlaylistPopupShow => {
+                if let Err(e) = self.mount_save_playlist() {
+                    self.mount_error_popup(e.context("mount save playlist"));
+                }
+                None
+            }
+            Msg::SavePlaylistPopupCloseCancel => {
+                self.umount_save_playlist();
+                None
+            }
+            Msg::SavePlaylistPopupCloseOk(filename) => {
+                self.umount_save_playlist();
+                if let Err(e) = self.playlist_save_m3u_before(&filename) {
+                    self.mount_error_popup(e.context("save m3u playlist before"));
+                }
+                None
+            }
+            Msg::SavePlaylistPopupUpdate(filename) => {
+                if let Err(e) = self.remount_save_playlist_label(&filename) {
+                    self.mount_error_popup(e.context("remount save playlist label"));
+                }
+                None
+            }
+            Msg::SavePlaylistConfirmCloseCancel => {
+                self.umount_save_playlist_confirm();
+                None
+            }
+            Msg::SavePlaylistConfirmCloseOk(filename) => {
+                if let Err(e) = self.playlist_save_m3u(Path::new(&filename)) {
+                    self.mount_error_popup(e.context("save m3u playlist"));
+                }
+                self.umount_save_playlist_confirm();
+                None
+            }
+            Msg::Podcast(m) => self.update_podcast(m),
+            Msg::LyricMessage(m) => self.update_lyric_textarea(m),
+            Msg::Download(m) => self.update_download_msg(&m),
+            Msg::Xywh(m) => self.update_xywh_msg(m),
+
+            Msg::ForceRedraw => None,
         }
     }
 }
@@ -196,7 +195,7 @@ impl Model {
             XYWHMsg::MoveDown => self.xywh_move_down(),
             XYWHMsg::ZoomIn => self.xywh_zoom_in(),
             XYWHMsg::ZoomOut => self.xywh_zoom_out(),
-            XYWHMsg::Hide => {
+            XYWHMsg::ToggleHidden => {
                 self.xywh_toggle_hide();
             }
         };
@@ -216,7 +215,7 @@ impl Model {
     }
 
     #[allow(clippy::too_many_lines)]
-    fn update_podcast(&mut self, msg: &PCMsg) -> Option<Msg> {
+    fn update_podcast(&mut self, msg: PCMsg) -> Option<Msg> {
         match msg {
             PCMsg::PodcastBlurDown => {
                 self.app.active(&Id::Episode).ok();
@@ -237,7 +236,7 @@ impl Model {
                 if url.starts_with("http") {
                     self.podcast_add(url);
                 } else {
-                    self.podcast_search_itunes(url);
+                    self.podcast_search_itunes(&url);
                     self.mount_podcast_search_table();
                 }
             }
@@ -250,7 +249,7 @@ impl Model {
                     None,
                     None,
                 );
-                if let Err(e) = self.add_or_sync_data(pod, Some(*id)) {
+                if let Err(e) = self.add_or_sync_data(&pod, Some(id)) {
                     self.mount_error_popup(e.context("add or sync data"));
                 };
             }
@@ -262,12 +261,12 @@ impl Model {
                     None,
                     None,
                 );
-                if let Err(e) = self.add_or_sync_data(pod, None) {
+                if let Err(e) = self.add_or_sync_data(&pod, None) {
                     self.mount_error_popup(e.context("add or sync data"));
                 }
             }
-            PCMsg::Error(url, feed) => {
-                self.download_tracker.decrease_one(url);
+            PCMsg::Error(feed) => {
+                self.download_tracker.decrease_one(&feed.url);
                 self.mount_error_popup(anyhow!("Error happened with feed: {:?}", feed.title));
                 self.show_message_timeout_label_help(
                     self.download_tracker.message_feed_sync_failed(),
@@ -277,19 +276,19 @@ impl Model {
                 );
             }
             PCMsg::PodcastSelected(index) => {
-                self.podcast.podcasts_index = *index;
+                self.podcast.podcasts_index = index;
                 if let Err(e) = self.podcast_sync_episodes() {
                     self.mount_error_popup(e.context("podcast sync episodes"));
                 }
             }
             PCMsg::DescriptionUpdate => self.lyric_update(),
             PCMsg::EpisodeAdd(index) => {
-                if let Err(e) = self.playlist_add_episode(*index) {
+                if let Err(e) = self.playlist_add_episode(index) {
                     self.mount_error_popup(e.context("podcast playlist add episode"));
                 }
             }
             PCMsg::EpisodeMarkPlayed(index) => {
-                if let Err(e) = self.episode_mark_played(*index) {
+                if let Err(e) = self.episode_mark_played(index) {
                     self.mount_error_popup(e.context("podcast episode mark played"));
                 }
             }
@@ -299,7 +298,7 @@ impl Model {
                 }
             }
             PCMsg::PodcastRefreshOne(index) => {
-                if let Err(e) = self.podcast_refresh_feeds(Some(*index)) {
+                if let Err(e) = self.podcast_refresh_feeds(Some(index)) {
                     self.mount_error_popup(e.context("podcast refresh feeds one"));
                 }
             }
@@ -318,7 +317,7 @@ impl Model {
                 );
             }
             PCMsg::EpisodeDownload(index) => {
-                if let Err(e) = self.episode_download(Some(*index)) {
+                if let Err(e) = self.episode_download(Some(index)) {
                     self.mount_error_popup(e.context("podcast episode download"));
                 }
             }
@@ -377,7 +376,7 @@ impl Model {
                 );
             }
             PCMsg::EpisodeDeleteFile(index) => {
-                if let Err(e) = self.episode_delete_file(*index) {
+                if let Err(e) = self.episode_delete_file(index) {
                     self.mount_error_popup(e.context("podcast episode delete"));
                 }
             }
@@ -400,9 +399,8 @@ impl Model {
             PCMsg::SearchItunesCloseCancel => self.umount_podcast_search_table(),
             PCMsg::SearchItunesCloseOk(index) => {
                 if let Some(vec) = &self.podcast.search_results {
-                    if let Some(pod) = vec.get(*index) {
-                        let url = pod.url.clone();
-                        self.podcast_add(&url);
+                    if let Some(pod) = vec.get(index) {
+                        self.podcast_add(pod.url.clone());
                     }
                 }
             }
@@ -410,7 +408,7 @@ impl Model {
                 self.podcast.search_results = Some(vec.clone());
                 self.update_podcast_search_table();
             }
-            PCMsg::SearchError(e) => self.mount_error_popup(anyhow!(e.to_owned())),
+            PCMsg::SearchError(e) => self.mount_error_popup(anyhow!(e)),
         }
         None
     }
@@ -429,7 +427,7 @@ impl Model {
                     );
                     return None;
                 }
-                self.command(&PlayerCmd::SeekForward);
+                self.command(TuiCmd::SeekForward);
             }
             Msg::PlayerSeekBackward => {
                 if self.is_radio() {
@@ -441,22 +439,22 @@ impl Model {
                     );
                     return None;
                 }
-                self.command(&PlayerCmd::SeekBackward);
+                self.command(TuiCmd::SeekBackward);
             }
             Msg::PlayerSpeedUp => {
-                self.command(&PlayerCmd::SpeedUp);
+                self.command(TuiCmd::SpeedUp);
             }
             Msg::PlayerSpeedDown => {
-                self.command(&PlayerCmd::SpeedDown);
+                self.command(TuiCmd::SpeedDown);
             }
             Msg::PlayerVolumeUp => {
-                self.command(&PlayerCmd::VolumeUp);
+                self.command(TuiCmd::VolumeUp);
             }
             Msg::PlayerVolumeDown => {
-                self.command(&PlayerCmd::VolumeDown);
+                self.command(TuiCmd::VolumeDown);
             }
             Msg::PlayerToggleGapless => {
-                self.command(&PlayerCmd::ToggleGapless);
+                self.command(TuiCmd::ToggleGapless);
             }
             _ => {}
         }
@@ -628,7 +626,7 @@ impl Model {
         }
     }
 
-    fn update_youtube_search(&mut self, msg: &YSMsg) {
+    fn update_youtube_search(&mut self, msg: YSMsg) {
         match msg {
             YSMsg::InputPopupShow => {
                 self.mount_youtube_search_input();
@@ -643,7 +641,7 @@ impl Model {
                     assert!(self.app.umount(&Id::YoutubeSearchInputPopup).is_ok());
                 }
                 if url.starts_with("http") {
-                    match self.youtube_dl(url) {
+                    match self.youtube_dl(&url) {
                         Ok(()) => {}
                         Err(e) => {
                             self.mount_error_popup(e.context("youtube-dl download"));
@@ -664,11 +662,20 @@ impl Model {
                 self.youtube_options_prev_page();
             }
             YSMsg::TablePopupCloseOk(index) => {
-                if let Err(e) = self.youtube_options_download(*index) {
+                if let Err(e) = self.youtube_options_download(index) {
                     self.library_reload_with_node_focus(None);
                     self.mount_error_popup(e.context("youtube-dl options download"));
                 }
             }
+            YSMsg::YoutubeSearchSuccess(youtube_options) => {
+                self.youtube_options = youtube_options;
+                self.sync_youtube_options();
+                self.redraw = true;
+            }
+            YSMsg::YoutubeSearchFail(e) => {
+                self.redraw = true;
+                self.mount_error_popup(anyhow!("Youtube search fail: {e}"));
+            }
         }
     }
 
@@ -837,7 +844,7 @@ impl Model {
                 self.playlist_play_selected(*index);
             }
             PLMsg::LoopModeCycle => {
-                self.command(&PlayerCmd::CycleLoop);
+                self.command(TuiCmd::CycleLoop);
                 self.config_server.write().settings.player.loop_mode =
                     self.playlist.cycle_loop_mode();
                 self.playlist_update_title();
@@ -848,7 +855,7 @@ impl Model {
                 TermusicLayout::Podcast => assert!(self.app.active(&Id::Lyric).is_ok()),
             },
             PLMsg::NextSong => {
-                self.command(&PlayerCmd::SkipNext);
+                self.command(TuiCmd::SkipNext);
             }
 
             PLMsg::PrevSong => {
@@ -901,23 +908,27 @@ impl Model {
         }
     }
 
+    /// Show a message with a `title` and `text`, and hide it again after `time_out` or 10 seconds.
+    ///
+    /// This function requires to run in a tokio context.
     pub fn update_show_message_timeout(&self, title: &str, text: &str, time_out: Option<u64>) {
         let title_string = title.to_string();
         let text_string = text.to_string();
         let tx = self.tx_to_main.clone();
-        thread::spawn(move || {
-            tx.send(Msg::Download(DLMsg::MessageShow((
+        let delay = time_out.unwrap_or(10);
+
+        Handle::current().spawn(async move {
+            let _ = tx.send(Msg::Download(DLMsg::MessageShow((
                 title_string.clone(),
                 text_string.clone(),
-            ))))
-            .expect("send first message error.");
-            let delay = time_out.unwrap_or(10);
-            sleep(Duration::from_secs(delay));
-            tx.send(Msg::Download(DLMsg::MessageHide((
+            ))));
+
+            sleep(Duration::from_secs(delay)).await;
+
+            let _ = tx.send(Msg::Download(DLMsg::MessageHide((
                 title_string,
                 text_string,
-            ))))
-            .expect("send second message error.");
+            ))));
         });
     }
 
@@ -952,7 +963,7 @@ impl Model {
         self.redraw = true;
         match msg {
             DLMsg::DownloadRunning(url, title) => {
-                self.download_tracker.increase_one(url);
+                self.download_tracker.increase_one(&**url);
                 self.show_message_timeout_label_help(
                     self.download_tracker.message_download_start(title),
                     None,
@@ -1005,14 +1016,6 @@ impl Model {
             DLMsg::MessageHide((title, text)) => {
                 self.umount_message(title, text);
             }
-            DLMsg::YoutubeSearchSuccess(y) => {
-                self.youtube_options = y.clone();
-                self.sync_youtube_options();
-                self.redraw = true;
-            }
-            DLMsg::YoutubeSearchFail(e) => {
-                self.mount_error_popup(anyhow!("Youtube search fail: {e}"));
-            }
             DLMsg::FetchPhotoSuccess(image_wrapper) => {
                 self.show_image(&image_wrapper.data).ok();
             }
