@@ -9,6 +9,7 @@ use crate::ui::utils::{
 };
 use crate::ui::Application;
 use anyhow::{bail, Result};
+use std::path::Path;
 use std::time::{Duration, Instant};
 use termusiclib::config::SharedTuiSettings;
 /**
@@ -39,14 +40,17 @@ use termusiclib::utils::get_parent_folder;
 use tui_realm_treeview::Tree;
 use tuirealm::event::NoUserEvent;
 use tuirealm::props::{AttrValue, Attribute, Color, PropPayload, PropValue, TextSpan};
-use tuirealm::tui::layout::{Constraint, Direction, Layout};
-use tuirealm::tui::widgets::Clear;
+use tuirealm::ratatui::layout::{Constraint, Direction, Layout};
+use tuirealm::ratatui::widgets::Clear;
 use tuirealm::EventListenerCfg;
 use tuirealm::{Frame, State, StateValue};
 
 impl Model {
     #[allow(clippy::too_many_lines)]
-    pub fn init_app(tree: &Tree, config: &SharedTuiSettings) -> Application<Id, Msg, NoUserEvent> {
+    pub fn init_app(
+        tree: &Tree<String>,
+        config: &SharedTuiSettings,
+    ) -> Application<Id, Msg, NoUserEvent> {
         // Setup application
         // NOTE: NoUserEvent is a shorthand to tell tui-realm we're not going to use any custom user event
         // NOTE: the event listener is configured to use the default crossterm input listener and to raise a Tick event each second
@@ -54,99 +58,12 @@ impl Model {
 
         let mut app: Application<Id, Msg, NoUserEvent> = Application::init(
             EventListenerCfg::default()
-                .default_input_listener(Duration::from_millis(20))
+                .crossterm_input_listener(Duration::from_millis(20), 20)
                 .poll_timeout(Duration::from_millis(10))
                 .tick_interval(Duration::from_secs(1)),
         );
-        assert!(app
-            .mount(
-                Id::Library,
-                Box::new(MusicLibrary::new(tree, None, config.clone())),
-                vec![]
-            )
-            .is_ok());
-        assert!(app
-            .mount(
-                Id::DBListCriteria,
-                Box::new(DBListCriteria::new(
-                    config.clone(),
-                    Msg::DataBase(DBMsg::CriteriaBlurDown),
-                    Msg::DataBase(DBMsg::CriteriaBlurUp)
-                )),
-                vec![]
-            )
-            .is_ok());
 
-        assert!(app
-            .mount(
-                Id::DBListSearchResult,
-                Box::new(DBListSearchResult::new(
-                    config.clone(),
-                    Msg::DataBase(DBMsg::SearchResultBlurDown),
-                    Msg::DataBase(DBMsg::SearchResultBlurUp)
-                )),
-                vec![]
-            )
-            .is_ok());
-        assert!(app
-            .mount(
-                Id::DBListSearchTracks,
-                Box::new(DBListSearchTracks::new(
-                    config.clone(),
-                    Msg::DataBase(DBMsg::SearchTracksBlurDown),
-                    Msg::DataBase(DBMsg::SearchTracksBlurUp)
-                )),
-                vec![]
-            )
-            .is_ok());
-        assert!(app
-            .mount(
-                Id::Playlist,
-                Box::new(Playlist::new(config.clone())),
-                vec![]
-            )
-            .is_ok());
-        assert!(app
-            .mount(
-                Id::Progress,
-                Box::new(Progress::new(&config.read())),
-                vec![]
-            )
-            .is_ok());
-        assert!(app
-            .mount(Id::Lyric, Box::new(Lyric::new(config.clone())), vec![])
-            .is_ok());
-
-        assert!(app
-            .mount(
-                Id::Podcast,
-                Box::new(FeedsList::new(
-                    config.clone(),
-                    Msg::Podcast(PCMsg::PodcastBlurDown),
-                    Msg::Podcast(PCMsg::PodcastBlurUp)
-                )),
-                vec![]
-            )
-            .is_ok());
-
-        assert!(app
-            .mount(
-                Id::Episode,
-                Box::new(EpisodeList::new(
-                    config.clone(),
-                    Msg::Podcast(PCMsg::EpisodeBlurDown),
-                    Msg::Podcast(PCMsg::EpisodeBlurUp)
-                )),
-                vec![]
-            )
-            .is_ok());
-        assert!(app
-            .mount(
-                Id::DownloadSpinner,
-                Box::new(DownloadSpinner::new(&config.read())),
-                vec![]
-            )
-            .is_ok());
+        Self::mount_main(&mut app, config, tree).unwrap();
 
         // Mount global hotkey listener
         assert!(app
@@ -161,6 +78,85 @@ impl Model {
         app
     }
 
+    /// Mount the Main components for the TUI
+    fn mount_main(
+        app: &mut Application<Id, Msg, NoUserEvent>,
+        config: &SharedTuiSettings,
+        tree: &Tree<String>,
+    ) -> Result<()> {
+        app.mount(
+            Id::Library,
+            Box::new(MusicLibrary::new(tree, None, config.clone())),
+            Vec::new(),
+        )?;
+        app.mount(
+            Id::DBListCriteria,
+            Box::new(DBListCriteria::new(
+                config.clone(),
+                Msg::DataBase(DBMsg::CriteriaBlurDown),
+                Msg::DataBase(DBMsg::CriteriaBlurUp),
+            )),
+            Vec::new(),
+        )?;
+
+        app.mount(
+            Id::DBListSearchResult,
+            Box::new(DBListSearchResult::new(
+                config.clone(),
+                Msg::DataBase(DBMsg::SearchResultBlurDown),
+                Msg::DataBase(DBMsg::SearchResultBlurUp),
+            )),
+            Vec::new(),
+        )?;
+        app.mount(
+            Id::DBListSearchTracks,
+            Box::new(DBListSearchTracks::new(
+                config.clone(),
+                Msg::DataBase(DBMsg::SearchTracksBlurDown),
+                Msg::DataBase(DBMsg::SearchTracksBlurUp),
+            )),
+            Vec::new(),
+        )?;
+        app.mount(
+            Id::Playlist,
+            Box::new(Playlist::new(config.clone())),
+            Vec::new(),
+        )?;
+        app.mount(
+            Id::Progress,
+            Box::new(Progress::new(&config.read())),
+            Vec::new(),
+        )?;
+        app.mount(Id::Lyric, Box::new(Lyric::new(config.clone())), Vec::new())?;
+
+        app.mount(
+            Id::Podcast,
+            Box::new(FeedsList::new(
+                config.clone(),
+                Msg::Podcast(PCMsg::PodcastBlurDown),
+                Msg::Podcast(PCMsg::PodcastBlurUp),
+            )),
+            Vec::new(),
+        )?;
+
+        app.mount(
+            Id::Episode,
+            Box::new(EpisodeList::new(
+                config.clone(),
+                Msg::Podcast(PCMsg::EpisodeBlurDown),
+                Msg::Podcast(PCMsg::EpisodeBlurUp),
+            )),
+            Vec::new(),
+        )?;
+        app.mount(
+            Id::DownloadSpinner,
+            Box::new(DownloadSpinner::new(&config.read())),
+            Vec::new(),
+        )?;
+
+        Ok(())
+    }
+
     pub fn view(&mut self) {
         if self.redraw {
             self.redraw = false;
@@ -204,7 +200,7 @@ impl Model {
                         ]
                         .as_ref(),
                     )
-                    .split(f.size());
+                    .split(f.area());
                 let chunks_center = Layout::default()
                     .direction(Direction::Horizontal)
                     .margin(0)
@@ -241,7 +237,7 @@ impl Model {
                     .direction(Direction::Vertical)
                     .margin(0)
                     .constraints([Constraint::Min(2), Constraint::Length(1)].as_ref())
-                    .split(f.size());
+                    .split(f.area());
                 let chunks_left = Layout::default()
                     .direction(Direction::Horizontal)
                     .margin(0)
@@ -296,7 +292,7 @@ impl Model {
                     .direction(Direction::Vertical)
                     .margin(0)
                     .constraints([Constraint::Min(2), Constraint::Length(1)].as_ref())
-                    .split(f.size());
+                    .split(f.area());
                 let chunks_left = Layout::default()
                     .direction(Direction::Horizontal)
                     .margin(0)
@@ -338,7 +334,7 @@ impl Model {
                 .direction(Direction::Vertical)
                 .margin(0)
                 .constraints([Constraint::Min(2), Constraint::Length(1)].as_ref())
-                .split(f.size());
+                .split(f.area());
             let chunks_footer = Layout::default()
                 .direction(Direction::Horizontal)
                 .margin(0)
@@ -359,37 +355,37 @@ impl Model {
                 .direction(Direction::Vertical)
                 .margin(0)
                 .constraints([Constraint::Min(2), Constraint::Length(1)].as_ref())
-                .split(f.size());
+                .split(f.area());
             app.view(&Id::Label, f, chunks_main[1]);
         }
 
         // -- popups
         if app.mounted(&Id::QuitPopup) {
-            let popup = draw_area_in_absolute(f.size(), 30, 3);
+            let popup = draw_area_in_absolute(f.area(), 30, 3);
             f.render_widget(Clear, popup);
             app.view(&Id::QuitPopup, f, popup);
         } else if app.mounted(&Id::HelpPopup) {
-            let popup = draw_area_in_relative(f.size(), 88, 91);
+            let popup = draw_area_in_relative(f.area(), 88, 91);
             f.render_widget(Clear, popup);
             app.view(&Id::HelpPopup, f, popup);
         } else if app.mounted(&Id::DeleteConfirmRadioPopup) {
-            let popup = draw_area_in_absolute(f.size(), 30, 3);
+            let popup = draw_area_in_absolute(f.area(), 30, 3);
             f.render_widget(Clear, popup);
             app.view(&Id::DeleteConfirmRadioPopup, f, popup);
         } else if app.mounted(&Id::DeleteConfirmInputPopup) {
-            let popup = draw_area_in_absolute(f.size(), 30, 3);
+            let popup = draw_area_in_absolute(f.area(), 72, 3);
             f.render_widget(Clear, popup);
             app.view(&Id::DeleteConfirmInputPopup, f, popup);
         } else if app.mounted(&Id::FeedDeleteConfirmRadioPopup) {
-            let popup = draw_area_in_absolute(f.size(), 60, 3);
+            let popup = draw_area_in_absolute(f.area(), 60, 3);
             f.render_widget(Clear, popup);
             app.view(&Id::FeedDeleteConfirmRadioPopup, f, popup);
         } else if app.mounted(&Id::FeedDeleteConfirmInputPopup) {
-            let popup = draw_area_in_absolute(f.size(), 60, 3);
+            let popup = draw_area_in_absolute(f.area(), 60, 3);
             f.render_widget(Clear, popup);
             app.view(&Id::FeedDeleteConfirmInputPopup, f, popup);
         } else if app.mounted(&Id::GeneralSearchInput) {
-            let popup = draw_area_in_relative(f.size(), 65, 68);
+            let popup = draw_area_in_relative(f.area(), 65, 68);
             f.render_widget(Clear, popup);
             let popup_chunks = Layout::default()
                 .direction(Direction::Vertical)
@@ -404,19 +400,19 @@ impl Model {
             app.view(&Id::GeneralSearchInput, f, popup_chunks[0]);
             app.view(&Id::GeneralSearchTable, f, popup_chunks[1]);
         } else if app.mounted(&Id::YoutubeSearchInputPopup) {
-            let popup = draw_area_in_absolute(f.size(), 50, 3);
+            let popup = draw_area_in_absolute(f.area(), 50, 3);
             f.render_widget(Clear, popup);
             app.view(&Id::YoutubeSearchInputPopup, f, popup);
         } else if app.mounted(&Id::YoutubeSearchTablePopup) {
-            let popup = draw_area_in_relative(f.size(), 65, 68);
+            let popup = draw_area_in_relative(f.area(), 65, 68);
             f.render_widget(Clear, popup);
             app.view(&Id::YoutubeSearchTablePopup, f, popup);
         } else if app.mounted(&Id::PodcastSearchTablePopup) {
-            let popup = draw_area_in_relative(f.size(), 65, 68);
+            let popup = draw_area_in_relative(f.area(), 65, 68);
             f.render_widget(Clear, popup);
             app.view(&Id::PodcastSearchTablePopup, f, popup);
         } else if app.mounted(&Id::SavePlaylistPopup) {
-            let popup = draw_area_in_absolute(f.size(), 76, 6);
+            let popup = draw_area_in_absolute(f.area(), 76, 6);
             f.render_widget(Clear, popup);
             let popup_chunks = Layout::default()
                 .direction(Direction::Vertical)
@@ -425,148 +421,78 @@ impl Model {
             app.view(&Id::SavePlaylistPopup, f, popup_chunks[0]);
             app.view(&Id::SavePlaylistLabel, f, popup_chunks[1]);
         } else if app.mounted(&Id::SavePlaylistConfirm) {
-            let popup = draw_area_in_absolute(f.size(), 40, 3);
+            let popup = draw_area_in_absolute(f.area(), 40, 3);
             f.render_widget(Clear, popup);
             app.view(&Id::SavePlaylistConfirm, f, popup);
         } else if app.mounted(&Id::PodcastAddPopup) {
-            let popup = draw_area_in_absolute(f.size(), 65, 3);
+            let popup = draw_area_in_absolute(f.area(), 65, 3);
             f.render_widget(Clear, popup);
             app.view(&Id::PodcastAddPopup, f, popup);
         }
         if app.mounted(&Id::MessagePopup) {
-            let popup = draw_area_top_right_absolute(f.size(), 25, 4);
+            let popup = draw_area_top_right_absolute(f.area(), 25, 4);
             f.render_widget(Clear, popup);
             app.view(&Id::MessagePopup, f, popup);
         }
         if app.mounted(&Id::ErrorPopup) {
-            let popup = draw_area_in_absolute(f.size(), 50, 4);
+            let popup = draw_area_in_absolute(f.area(), 50, 4);
             f.render_widget(Clear, popup);
             app.view(&Id::ErrorPopup, f, popup);
         }
     }
 
+    /// Mount / Remount a search popup for the provided source
+    fn mount_search(&mut self, source: Source) {
+        self.app
+            .remount(
+                Id::GeneralSearchInput,
+                Box::new(GSInputPopup::new(source, &self.config_tui.read())),
+                Vec::new(),
+            )
+            .unwrap();
+        self.app
+            .remount(
+                Id::GeneralSearchTable,
+                Box::new(GSTablePopup::new(source, self.config_tui.clone())),
+                Vec::new(),
+            )
+            .unwrap();
+
+        self.app.active(&Id::GeneralSearchInput).unwrap();
+        if let Err(e) = self.update_photo() {
+            self.mount_error_popup(e.context("update_photo"));
+        }
+    }
+
+    #[inline]
     pub fn mount_search_library(&mut self) {
-        assert!(self
-            .app
-            .remount(
-                Id::GeneralSearchInput,
-                Box::new(GSInputPopup::new(Source::Library, &self.config_tui.read())),
-                vec![]
-            )
-            .is_ok());
-        assert!(self
-            .app
-            .remount(
-                Id::GeneralSearchTable,
-                Box::new(GSTablePopup::new(Source::Library, self.config_tui.clone())),
-                vec![]
-            )
-            .is_ok());
-
-        assert!(self.app.active(&Id::GeneralSearchInput).is_ok());
-        if let Err(e) = self.update_photo() {
-            self.mount_error_popup(e.context("update_photo"));
-        }
+        self.mount_search(Source::Library);
     }
 
+    #[inline]
     pub fn mount_search_playlist(&mut self) {
-        assert!(self
-            .app
-            .remount(
-                Id::GeneralSearchInput,
-                Box::new(GSInputPopup::new(Source::Playlist, &self.config_tui.read())),
-                vec![]
-            )
-            .is_ok());
-        assert!(self
-            .app
-            .remount(
-                Id::GeneralSearchTable,
-                Box::new(GSTablePopup::new(Source::Playlist, self.config_tui.clone())),
-                vec![]
-            )
-            .is_ok());
-        assert!(self.app.active(&Id::GeneralSearchInput).is_ok());
-        if let Err(e) = self.update_photo() {
-            self.mount_error_popup(e.context("update_photo"));
-        }
+        self.mount_search(Source::Playlist);
     }
 
+    #[inline]
     pub fn mount_search_database(&mut self) {
-        assert!(self
-            .app
-            .remount(
-                Id::GeneralSearchInput,
-                Box::new(GSInputPopup::new(Source::Database, &self.config_tui.read())),
-                vec![]
-            )
-            .is_ok());
-        assert!(self
-            .app
-            .remount(
-                Id::GeneralSearchTable,
-                Box::new(GSTablePopup::new(Source::Database, self.config_tui.clone())),
-                vec![]
-            )
-            .is_ok());
-        assert!(self.app.active(&Id::GeneralSearchInput).is_ok());
-        if let Err(e) = self.update_photo() {
-            self.mount_error_popup(e.context("update_photo"));
-        }
+        self.mount_search(Source::Database);
     }
 
+    #[inline]
     pub fn mount_search_episode(&mut self) {
-        assert!(self
-            .app
-            .remount(
-                Id::GeneralSearchInput,
-                Box::new(GSInputPopup::new(Source::Episode, &self.config_tui.read())),
-                vec![]
-            )
-            .is_ok());
-        assert!(self
-            .app
-            .remount(
-                Id::GeneralSearchTable,
-                Box::new(GSTablePopup::new(Source::Episode, self.config_tui.clone())),
-                vec![]
-            )
-            .is_ok());
-
-        assert!(self.app.active(&Id::GeneralSearchInput).is_ok());
-        if let Err(e) = self.update_photo() {
-            self.mount_error_popup(e.context("update_photo"));
-        }
+        self.mount_search(Source::Episode);
     }
 
+    #[inline]
     pub fn mount_search_podcast(&mut self) {
-        assert!(self
-            .app
-            .remount(
-                Id::GeneralSearchInput,
-                Box::new(GSInputPopup::new(Source::Podcast, &self.config_tui.read())),
-                vec![]
-            )
-            .is_ok());
-        assert!(self
-            .app
-            .remount(
-                Id::GeneralSearchTable,
-                Box::new(GSTablePopup::new(Source::Podcast, self.config_tui.clone())),
-                vec![]
-            )
-            .is_ok());
-
-        assert!(self.app.active(&Id::GeneralSearchInput).is_ok());
-        if let Err(e) = self.update_photo() {
-            self.mount_error_popup(e.context("update_photo"));
-        }
+        self.mount_search(Source::Podcast);
     }
 
     pub fn mount_label_help(&mut self) {
         let config = self.config_tui.read();
         self.app
-            .remount(Id::Label, Box::new(Footer::new(&config)), Vec::default())
+            .remount(Id::Label, Box::new(Footer::new(&config)), Vec::new())
             .expect("Expected to remount without error");
     }
 
@@ -576,7 +502,10 @@ impl Model {
             _ => bail!("Invalid node selected in library"),
         };
 
-        let mut path_string = get_parent_folder(&current_node);
+        let mut path_string = get_parent_folder(Path::new(&current_node))
+            .to_string_lossy()
+            .to_string();
+        // push extra "/" as "Path::to_string()" does not end with a "/"
         path_string.push('/');
 
         let config = self.config_tui.read();
@@ -599,7 +528,7 @@ impl Model {
                             .bold(),
                     ],
                 )),
-                Vec::default(),
+                Vec::new(),
             )
             .expect("Expected to remount without error");
         Ok(())
