@@ -28,12 +28,13 @@ use id3::Version::Id3v24;
 use lazy_static::lazy_static;
 use regex::Regex;
 use std::path::{Path, PathBuf};
+use std::sync::Arc;
 use std::thread::{self, sleep};
 use std::time::Duration;
 use termusiclib::invidious::Instance;
 use termusiclib::track::Track;
-use termusiclib::types::YoutubeOptions;
 use termusiclib::types::{DLMsg, Id, Msg};
+use termusiclib::types::{YSMsg, YoutubeOptions};
 use termusiclib::utils::get_parent_folder;
 use tokio::runtime::Handle;
 use tuirealm::props::{Alignment, AttrValue, Attribute, TableBuilder, TextSpan};
@@ -60,22 +61,23 @@ impl Model {
     }
 
     /// This function requires to be run in a tokio Runtime context
-    pub fn youtube_options_search(&mut self, keyword: &str) {
-        let search_word = keyword.to_string();
+    pub fn youtube_options_search(&mut self, keyword: String) {
         let tx = self.tx_to_main.clone();
         tokio::spawn(async move {
-            match Instance::new(&search_word).await {
+            match Instance::new(&keyword).await {
                 Ok((instance, result)) => {
                     let youtube_options = YoutubeOptions {
                         items: result,
                         page: 1,
                         invidious_instance: instance,
                     };
-                    tx.send(Msg::Download(DLMsg::YoutubeSearchSuccess(youtube_options)))
-                        .ok();
+                    tx.send(Msg::YoutubeSearch(YSMsg::YoutubeSearchSuccess(
+                        youtube_options,
+                    )))
+                    .ok();
                 }
                 Err(e) => {
-                    tx.send(Msg::Download(DLMsg::YoutubeSearchFail(e.to_string())))
+                    tx.send(Msg::YoutubeSearch(YSMsg::YoutubeSearchFail(e.to_string())))
                         .ok();
                 }
             }
@@ -134,8 +136,7 @@ impl Model {
                 table.add_row();
             }
             let duration =
-                Track::duration_formatted_short(&Duration::from_secs(record.length_seconds))
-                    .to_string();
+                Track::duration_formatted_short(&Duration::from_secs(record.length_seconds));
             let duration_string = format!("[{duration:^10.10}]");
 
             let title = record.title.as_str();
@@ -174,7 +175,7 @@ impl Model {
     pub fn youtube_dl(&mut self, url: &str) -> Result<()> {
         let mut path: PathBuf = std::env::temp_dir();
         if let Ok(State::One(StateValue::String(node_id))) = self.app.state(&Id::Library) {
-            path = PathBuf::from(get_parent_folder(&node_id));
+            path = get_parent_folder(Path::new(&node_id)).to_path_buf();
         }
         let args = vec![
             Arg::new("--extract-audio"),
@@ -183,6 +184,8 @@ impl Model {
             Arg::new("--add-metadata"),
             Arg::new("--embed-thumbnail"),
             Arg::new_with_arg("--metadata-from-title", "%(artist) - %(title)s"),
+            #[cfg(target_os = "windows")]
+            Arg::new("--restrict-filenames"),
             Arg::new("--write-sub"),
             Arg::new("--all-subs"),
             Arg::new_with_arg("--convert-subs", "lrc"),
@@ -191,7 +194,9 @@ impl Model {
 
         let ytd = YoutubeDL::new(&path, args, url)?;
         let tx = self.tx_to_main.clone();
-        let url = url.to_string();
+
+        // avoid full string clones when sending via a channel
+        let url: Arc<str> = Arc::from(url);
 
         thread::spawn(move || -> Result<()> {
             tx.send(Msg::Download(DLMsg::DownloadRunning(
@@ -213,7 +218,7 @@ impl Model {
                         extract_filepath(result.output(), &path.to_string_lossy())
                     {
                         tx.send(Msg::Download(DLMsg::DownloadCompleted(
-                            url.to_string(),
+                            url,
                             Some(file_fullname.clone()),
                         )))
                         .ok();
@@ -223,7 +228,7 @@ impl Model {
 
                         embed_downloaded_lrc(&path, &file_fullname);
                     } else {
-                        tx.send(Msg::Download(DLMsg::DownloadCompleted(url.clone(), None)))
+                        tx.send(Msg::Download(DLMsg::DownloadCompleted(url, None)))
                             .ok();
                     }
                 }
@@ -235,7 +240,7 @@ impl Model {
                     )))
                     .ok();
                     sleep(Duration::from_secs(5));
-                    tx.send(Msg::Download(DLMsg::DownloadCompleted(url.clone(), None)))
+                    tx.send(Msg::Download(DLMsg::DownloadCompleted(url, None)))
                         .ok();
                 }
             }
@@ -272,18 +277,16 @@ fn remove_downloaded_json(path: &Path, file_fullname: &str) {
         .into_iter()
         .filter_map(std::result::Result::ok)
         .filter(|f| {
-            let name = f.file_name();
-            let p = Path::new(&name);
-            p.extension().map_or(false, |ext| ext == "json")
+            let p = Path::new(f.file_name());
+            p.extension().is_some_and(|ext| ext == "json")
         })
         .filter(|f| {
             let path_json = Path::new(f.file_name());
             let p1: &Path = Path::new(file_fullname);
-            path_json.file_stem().map_or(false, |stem_lrc| {
-                p1.file_stem().map_or(false, |p_base| {
+            path_json.file_stem().is_some_and(|stem_lrc| {
+                p1.file_stem().is_some_and(|p_base| {
                     stem_lrc
                         .to_string_lossy()
-                        .to_string()
                         .contains(p_base.to_string_lossy().as_ref())
                 })
             })
@@ -297,41 +300,40 @@ fn embed_downloaded_lrc(path: &Path, file_fullname: &str) {
     let mut id3_tag = if let Ok(tag) = id3::Tag::read_from_path(file_fullname) {
         tag
     } else {
-        let mut t = id3::Tag::new();
-        let p: &Path = Path::new(file_fullname);
-        if let Some(p_base) = p.file_stem() {
-            t.set_title(p_base.to_string_lossy());
+        let mut tags = id3::Tag::new();
+        let file_path = Path::new(file_fullname);
+        if let Some(p_base) = file_path.file_stem() {
+            tags.set_title(p_base.to_string_lossy());
         }
-        t.write_to_path(p, Id3v24).ok();
-        t
+        tags.write_to_path(file_path, Id3v24).ok();
+        tags
     };
 
     // here we add all downloaded lrc file
     let files = walkdir::WalkDir::new(path).follow_links(true);
 
-    for f in files
+    for entry in files
         .into_iter()
         .filter_map(std::result::Result::ok)
         .filter(|f| f.file_type().is_file())
         .filter(|f| {
             let name = f.file_name();
             let p = Path::new(&name);
-            p.extension().map_or(false, |ext| ext == "lrc")
+            p.extension().is_some_and(|ext| ext == "lrc")
         })
         .filter(|f| {
             let path_lrc = Path::new(f.file_name());
             let p1: &Path = Path::new(file_fullname);
-            path_lrc.file_stem().map_or(false, |stem_lrc| {
-                p1.file_stem().map_or(false, |p_base| {
+            path_lrc.file_stem().is_some_and(|stem_lrc| {
+                p1.file_stem().is_some_and(|p_base| {
                     stem_lrc
                         .to_string_lossy()
-                        .to_string()
                         .contains(p_base.to_string_lossy().as_ref())
                 })
             })
         })
     {
-        let path_lrc = Path::new(f.file_name());
+        let path_lrc = Path::new(entry.file_name());
         let mut lang_ext = "eng".to_string();
         if let Some(p_short) = path_lrc.file_stem() {
             let p2 = Path::new(p_short);
@@ -339,20 +341,19 @@ fn embed_downloaded_lrc(path: &Path, file_fullname: &str) {
                 lang_ext = ext2.to_string_lossy().to_string();
             }
         }
-        let lyric_string = std::fs::read_to_string(f.path());
+        let lyric_string = std::fs::read_to_string(entry.path());
         id3_tag.add_frame(id3::frame::Lyrics {
             lang: "eng".to_string(),
             description: lang_ext,
             text: lyric_string.unwrap_or_else(|_| String::from("[00:00:01] No lyric")),
         });
-        std::fs::remove_file(f.path()).ok();
+        std::fs::remove_file(entry.path()).ok();
     }
 
     id3_tag.write_to_path(file_fullname, Id3v24).ok();
 }
 
 #[cfg(test)]
-#[allow(clippy::non_ascii_literal)]
 mod tests {
 
     use crate::ui::model::youtube_options::extract_filepath;
